# Volatility
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

"""
@author: Edwin Smulders
@license: GNU General Public License 2.0 or later
@contact: mail@edwinsmulders.eu
"""

import volatility.plugins.linux.pslist as linux_pslist
from volatility.renderers.basic import Address
from volatility.renderers import TreeGrid

class linux_threads(linux_pslist.linux_pslist):
    """ Prints threads of processes """

    def unified_output(self, data):
        return TreeGrid([("Offset",Address),
                        ("NameProc",str),
                         ("TGID",int),
                         ("ThreadPid",str),
                            ("ThreadName", str),
                        ("thread_offset",Address),
                            ("Addr_limit",Address),
                            ("uid_cred",int),
                            ("gid_cred",int),
                            ("euid_cred",int)
                        ],
                        self.generator(data))

    def generator(self, data):

        for task in data:
            euidcred = task.euid
            uidcred = task.uid
            gidcred = task.gid
            for thread in task.threads():
                addr_limit = self.get_addr_limit(thread)
                yield(0,[Address(task.obj_offset),
                         str(task.comm),
                         int(task.tgid),
                         str(thread.pid),
                         str(thread.comm),
                         Address(thread.obj_offset),
                         Address(addr_limit),
                         int(uidcred),
                         int(gidcred),
                         int(euidcred)
                ])

    def get_addr_limit(self,thread, addrvar_offset = 8 ):
        """
        Here we read the addr_limit variable of a thread by reading at the offset of the thread plus
        the offset of the addr_limit variable inside the thread_info
        :param thread: thread from which we want the information
        :param addrvar_offset: offset of the addr_limit var in the thread_info
        :return: the addr_limit
        """
        addr_space = thread.get_process_address_space()
        offset = thread.obj_offset + addrvar_offset
        return addr_space.read_long_phys(offset)

    def render_text(self, outfd, data):
        for task in data:
            outfd.write("\nProcess Name: {}\nProcess ID: {}\n".format(task.comm, task.tgid))
            self.table_header(outfd, [('Thread PID', '13'), ('Thread Name', '16')])
            for thread in task.threads():
                self.table_row(outfd, str(thread.pid), thread.comm)

    





# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

# File-wide pylint filter for protected members, since we have three _BLAH structures
#pylint: disable-msg=W0212

import os
import volatility.utils as utils
import volatility.obj as obj
import volatility.debug as debug
import volatility.win32.tasks as tasks
import volatility.win32.modules as modules
import volatility.plugins.taskmods as taskmods
import volatility.plugins.vadinfo as vadinfo
import volatility.plugins.overlays.windows.windows as windows
import volatility.constants as constants
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address, Bytes

try:
    import yara
    has_yara = True
except ImportError:
    has_yara = False

try:
    import distorm3
    has_distorm3 = True
except ImportError:
    has_distorm3 = False

#--------------------------------------------------------------------------------
# functions 
#--------------------------------------------------------------------------------

def Disassemble(data, start, bits = '32bit', stoponret = False):
    """Dissassemble code with distorm3. 

    @param data: python byte str to decode
    @param start: address where `data` is found in memory
    @param bits: use 32bit or 64bit decoding 
    @param stoponret: stop disasm when function end is reached
    
    @returns: tuple of (offset, instruction, hex bytes)
    """

    if not has_distorm3:
        raise StopIteration

    if bits == '32bit':
        mode = distorm3.Decode32Bits
    else:
        mode = distorm3.Decode64Bits

    for o, _, i, h in distorm3.DecodeGenerator(start, data, mode):
        if stoponret and i.startswith("RET"):
            raise StopIteration
        yield o, i, h

#--------------------------------------------------------------------------------
# scanners by scudette
#
# unfortunately the existing scanning framework (i.e. scan.BaseScanner) has 
# some shortcomings that don't allow us to integrate yara easily. 
#
# FIXME: these may need updating after resolving issue 310 which aims to 
# enhance the scan.BaseScanner to better support things like this
#--------------------------------------------------------------------------------

class BaseYaraScanner(object):
    """An address space scanner for Yara signatures."""
    overlap = 1024

    def __init__(self, address_space = None, rules = None):
        self.rules = rules
        self.address_space = address_space

    def scan(self, offset, maxlen):
        # Start scanning from offset until maxlen:
        i = offset
        
        if isinstance(self.rules, list):
            rules = self.rules
        else:
            rules = [self.rules]

        while i < offset + maxlen:
            # Read some data and match it.
            to_read = min(constants.SCAN_BLOCKSIZE + self.overlap, offset + maxlen - i)
            data = self.address_space.zread(i, to_read)
            if data:
                for rule in rules:
                    for match in rule.match(data = data):
                        # We currently don't use name or value from the 
                        # yara results but they can be yielded in the 
                        # future if necessary. 
                        for moffset, _name, _value in match.strings:
                            if moffset < constants.SCAN_BLOCKSIZE:
                                yield match, moffset + i

            i += constants.SCAN_BLOCKSIZE

class VadYaraScanner(BaseYaraScanner):
    """A scanner over all memory regions of a process."""

    def __init__(self, task = None, **kwargs):
        """Scan the process address space through the Vads.

        Args:
          task: The _EPROCESS object for this task.
        """
        self.task = task
        BaseYaraScanner.__init__(self, address_space = task.get_process_address_space(), **kwargs)

    def scan(self, offset = 0, maxlen = None):
        for vad, self.address_space in self.task.get_vads(skip_max_commit = True):
            for match in BaseYaraScanner.scan(self, vad.Start, vad.Length):
                yield match

class DiscontigYaraScanner(BaseYaraScanner):
    """A Scanner for Discontiguous scanning."""

    def scan(self, start_offset = 0, maxlen = None):
        contiguous_offset = 0
        total_length = 0
        for (offset, length) in self.address_space.get_available_addresses():
            # Skip ranges before the start_offset
            if self.address_space.address_compare(offset, start_offset) == -1:
                continue

            # Skip ranges that are too high (if maxlen is specified)
            if maxlen != None:
                if self.address_space.address_compare(offset, start_offset + maxlen) > 0:
                    continue

            # Try to join up adjacent pages as much as possible.
            if offset == contiguous_offset + total_length:
                total_length += length
            else:
                # Scan the last contiguous range.
                for match in BaseYaraScanner.scan(self, contiguous_offset, total_length):
                    yield match

                # Reset the contiguous range.
                contiguous_offset = offset
                total_length = length

        if total_length > 0:
            # Do the last range.
            for match in BaseYaraScanner.scan(self, contiguous_offset, total_length):
                yield match

#--------------------------------------------------------------------------------
# yarascan
#--------------------------------------------------------------------------------

class YaraScan(taskmods.DllList):
    "Scan process or kernel memory with Yara signatures"

    def __init__(self, config, *args, **kwargs):
        taskmods.DllList.__init__(self, config, *args, **kwargs)
        config.add_option("ALL", short_option = 'A', default = False, action = 'store_true',
                        help = 'Scan both process and kernel memory')                
        config.add_option("CASE", short_option = 'C', default = False, action = 'store_true',
                        help = 'Make the search case insensitive')        
        config.add_option("KERNEL", short_option = 'K', default = False, action = 'store_true',
                        help = 'Scan kernel modules')
        config.add_option("WIDE", short_option = 'W', default = False, action = 'store_true',
                        help = 'Match wide (unicode) strings')
        config.add_option('YARA-RULES', short_option = 'Y', default = None,
                        help = 'Yara rules (as a string)')
        config.add_option('YARA-FILE', short_option = 'y', default = None,
                        help = 'Yara rules (rules file)')
        config.add_option('DUMP-DIR', short_option = 'D', default = None,
                        help = 'Directory in which to dump the files')
        config.add_option('SIZE', short_option = 's', default = 256,
                          help = 'Size of preview hexdump (in bytes)',
                          action = 'store', type = 'int')
        config.add_option('REVERSE', short_option = 'R', default = 0,
                          help = 'Reverse this number of bytes',
                          action = 'store', type = 'int')

    def _compile_rules(self):
        """Compile the YARA rules from command-line parameters. 
        
        @returns: a YARA object on which you can call 'match'
        
        This function causes the plugin to exit if the YARA 
        rules have syntax errors or are not supplied correctly. 
        """
    
        rules = None
    
        try:
            if self._config.YARA_RULES:
                s = self._config.YARA_RULES
                # Don't wrap hex or regex rules in quotes 
                if s[0] not in ("{", "/"): s = '"' + s + '"'
                # Option for case insensitive searches
                if self._config.CASE: s += " nocase"
                # Scan for unicode and ascii strings 
                if self._config.WIDE: s += " wide ascii"
                rules = yara.compile(sources = {
                            'n' : 'rule r1 {strings: $a = ' + s + ' condition: $a}'
                            })
            elif self._config.YARA_FILE and os.path.isfile(self._config.YARA_FILE):
                rules = yara.compile(self._config.YARA_FILE)
            else:
                debug.error("You must specify a string (-Y) or a rules file (-y)")
        except yara.SyntaxError, why:
            debug.error("Cannot compile rules: {0}".format(str(why)))
            
        return rules

    def _scan_process_memory(self, addr_space, rules):
        for task in self.filter_tasks(tasks.pslist(addr_space)):
            scanner = VadYaraScanner(task = task, rules = rules)
            for hit, address in scanner.scan():
                yield (task, address, hit, scanner.address_space.zread(address - self._config.REVERSE, self._config.SIZE))

    def _scan_kernel_memory(self, addr_space, rules):
        # Find KDBG so we know where kernel memory begins. Do not assume
        # the starting range is 0x80000000 because we may be dealing with
        # an image with the /3GB boot switch. 
        kdbg = tasks.get_kdbg(addr_space)

        start = kdbg.MmSystemRangeStart.dereference_as("Pointer")

        # Modules so we can map addresses to owners
        mods = dict((addr_space.address_mask(mod.DllBase), mod)
                    for mod in modules.lsmod(addr_space))
        mod_addrs = sorted(mods.keys())

        # There are multiple views (GUI sessions) of kernel memory.
        # Since we're scanning virtual memory and not physical, 
        # all sessions must be scanned for full coverage. This 
        # really only has a positive effect if the data you're
        # searching for is in GUI memory. 
        sessions = []

        for proc in tasks.pslist(addr_space):
            sid = proc.SessionId
            # Skip sessions we've already seen 
            if sid == None or sid in sessions:
                continue

            session_space = proc.get_process_address_space()
            if session_space == None:
                continue

            sessions.append(sid)
            scanner = DiscontigYaraScanner(address_space = session_space,
                                           rules = rules)

            for hit, address in scanner.scan(start_offset = start):
                module = tasks.find_module(mods, mod_addrs, addr_space.address_mask(address))
                yield (module, address, hit, session_space.zread(address - self._config.REVERSE, self._config.SIZE))  

    def calculate(self):
        if not has_yara:
            debug.error("Please install Yara from https://plusvic.github.io/yara/")

        addr_space = utils.load_as(self._config)
        rules = self._compile_rules()
        process_mem = self._scan_process_memory(addr_space, rules)
        kernel_mem = self._scan_kernel_memory(addr_space, rules)

        if self._config.ALL:
            for p in process_mem:
                yield p
            for k in kernel_mem:
                yield k
        elif self._config.KERNEL:
            for k in kernel_mem:
                yield k
        else:
            for p in process_mem:
                yield p

    def unified_output(self, data):
        return TreeGrid([("Rule", str),
                       ("Owner", str),
                       ("Address", Address),
                       ("Data", Bytes)],
                        self.generator(data))

    def generator(self, data):
        if self._config.DUMP_DIR and not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")
        for o, addr, hit, content in data:
            owner = "Owner: (Unknown Kernel Memory)"
            if o == None:
                filename = "kernel.{0:#x}.dmp".format(addr)
            elif o.obj_name == "_EPROCESS":
                owner = "{0}: (Pid {1})".format(o.ImageFileName, o.UniqueProcessId)
                filename = "process.{0:#x}.{1:#x}.dmp".format(o.obj_offset, addr)
            else:
                owner = "{0}".format(o.BaseDllName)
                filename = "kernel.{0:#x}.{1:#x}.dmp".format(o.obj_offset, addr)

            # Dump the data if --dump-dir was supplied
            if self._config.DUMP_DIR:
                path = os.path.join(self._config.DUMP_DIR, filename)
                fh = open(path, "wb")
                fh.write(content)
                fh.close()

            yield (0, [str(hit.rule), owner, Address(addr), Bytes(content)])

    def render_text(self, outfd, data):

        if self._config.DUMP_DIR and not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")
        for o, addr, hit, content in data:
            outfd.write("Rule: {0}\n".format(hit.rule))

            # Find out if the hit is from user or kernel mode
            if o == None:
                outfd.write("Owner: (Unknown Kernel Memory)\n")
                filename = "kernel.{0:#x}.dmp".format(addr)
            elif o.obj_name == "_EPROCESS":
                outfd.write("Owner: Process {0} Pid {1}\n".format(o.ImageFileName,
                    o.UniqueProcessId))
                filename = "process.{0:#x}.{1:#x}.dmp".format(o.obj_offset, addr)
            else:
                outfd.write("Owner: {0}\n".format(o.BaseDllName))
                filename = "kernel.{0:#x}.{1:#x}.dmp".format(o.obj_offset, addr)

            # Dump the data if --dump-dir was supplied
            if self._config.DUMP_DIR:
                path = os.path.join(self._config.DUMP_DIR, filename)
                fh = open(path, "wb")
                fh.write(content)
                fh.close()

            outfd.write("".join(
                ["{0:#010x}  {1:<48}  {2}\n".format(addr + o, h, ''.join(c))
                for o, h, c in utils.Hexdump(content)
                ]))

#--------------------------------------------------------------------------------
# malfind
#--------------------------------------------------------------------------------

class Malfind(vadinfo.VADDump):
    "Find hidden and injected code"

    def __init__(self, config, *args, **kwargs):
        vadinfo.VADDump.__init__(self, config, *args, **kwargs)
        config.remove_option("BASE")

    def _is_vad_empty(self, vad, address_space):
        """
        Check if a VAD region is either entirely unavailable 
        due to paging, entirely consiting of zeros, or a 
        combination of the two. This helps ignore false positives
        whose VAD flags match task._injection_filter requirements
        but there's no data and thus not worth reporting it. 

        @param vad: an MMVAD object in kernel AS
        @param address_space: the process address space 
        """
        
        PAGE_SIZE = 0x1000
        all_zero_page = "\x00" * PAGE_SIZE

        offset = 0
        while offset < vad.Length:
            next_addr = vad.Start + offset
            if (address_space.is_valid_address(next_addr) and 
                    address_space.read(next_addr, PAGE_SIZE) != all_zero_page):
                return False
            offset += PAGE_SIZE

        return True

    def unified_output(self, data):
        return TreeGrid([("Process", str),
                       ("Pid", int),
                       ("Address", Address),
                       ("VadTag", str),
                       ("Protection", str),
                       ("Flags", str),
                       ("Data", Bytes)],
                        self.generator(data))

    def generator(self, data):
        if self._config.DUMP_DIR and not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        for task in data:
            for vad, address_space in task.get_vads(vad_filter = task._injection_filter):

                if self._is_vad_empty(vad, address_space):
                    continue

                content = address_space.zread(vad.Start, 64) 

                yield (0, [str(task.ImageFileName), 
                           int(task.UniqueProcessId),
                           Address(vad.Start),
                           str(vad.Tag),
                           str(vadinfo.PROTECT_FLAGS.get(vad.VadFlags.Protection.v(), "")),
                           str(vad.VadFlags),
                           Bytes(content)])

                # Dump the data if --dump-dir was supplied
                if self._config.DUMP_DIR:

                    filename = os.path.join(self._config.DUMP_DIR,
                        "process.{0:#x}.{1:#x}.dmp".format(
                        task.obj_offset, vad.Start))

                    self.dump_vad(filename, vad, address_space)

    def render_text(self, outfd, data):

        if not has_distorm3:
            debug.warning("For best results please install distorm3")

        if self._config.DUMP_DIR and not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        for task in data:
            for vad, address_space in task.get_vads(vad_filter = task._injection_filter):

                if self._is_vad_empty(vad, address_space):
                    continue

                content = address_space.zread(vad.Start, 64)

                outfd.write("Process: {0} Pid: {1} Address: {2:#x}\n".format(
                    task.ImageFileName, task.UniqueProcessId, vad.Start))

                outfd.write("Vad Tag: {0} Protection: {1}\n".format(
                    vad.Tag, vadinfo.PROTECT_FLAGS.get(vad.VadFlags.Protection.v(), "")))

                outfd.write("Flags: {0}\n".format(str(vad.VadFlags)))
                outfd.write("\n")

                # this is for address reporting in the output 
                data_start = vad.Start

                # all zeros in the first page followed by 558B at the base of
                # the second page is an indicator of wiped PE headers
                if content.count(chr(0)) == len(content):
                    if address_space.zread(vad.Start, 0x1000).count(chr(0)) == 0x1000:
                        next_page = address_space.zread(vad.Start + 0x1000, 64)
                        if next_page[0:2] == "\x55\x8B":
                            outfd.write("**** POSSIBLE WIPED PE HEADER AT BASE *****\n\n")
                            content = next_page    
                            data_start = vad.Start + 0x1000                    

                outfd.write("{0}\n".format("\n".join(
                    ["{0:#010x}  {1:<48}  {2}".format(data_start + o, h, ''.join(c))
                    for o, h, c in utils.Hexdump(content)
                    ])))

                outfd.write("\n")
                outfd.write("\n".join(
                    ["{0:#010x} {1:<16} {2}".format(o, h, i)
                    for o, i, h in Disassemble(content, data_start)
                    ]))

                # Dump the data if --dump-dir was supplied
                if self._config.DUMP_DIR:

                    filename = os.path.join(self._config.DUMP_DIR,
                        "process.{0:#x}.{1:#x}.dmp".format(
                        task.obj_offset, vad.Start))

                    self.dump_vad(filename, vad, address_space)

                outfd.write("\n\n")

#--------------------------------------------------------------------------------
# ldrmodules 
#--------------------------------------------------------------------------------

class LdrModules(taskmods.DllList):
    "Detect unlinked DLLs"

    def unified_output(self, data):
        if self._config.verbose:
            return TreeGrid([("Pid", int),
                        ("Process", str), 
                        ("Base", Address),
                        ("InLoad", str),
                        ("InInit", str),
                        ("InMem", str),
                        ("MappedPath", str),
                        ("LoadPath", str),
                        ("InitPath", str),
                        ("MemPath", str)],
                        self.generator(data))
        else:
            return TreeGrid([("Pid", int),
                        ("Process", str),
                        ("Base", Address),
                        ("InLoad", str),
                        ("InInit", str),
                        ("InMem", str),
                        ("MappedPath", str)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            inloadorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_load_modules())
            ininitorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_init_modules())
            inmemorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_mem_modules())

            # Build a similar dictionary for the mapped files 
            mapped_files = {}
            for vad, address_space in task.get_vads(vad_filter = task._mapped_file_filter):
                # Note this is a lot faster than acquiring the full
                # vad region and then checking the first two bytes. 
                if obj.Object("_IMAGE_DOS_HEADER", offset = vad.Start, vm = address_space).e_magic != 0x5A4D:
                    continue
                mapped_files[int(vad.Start)] = str(vad.FileObject.FileName or '')

            # For each base address with a mapped file, print info on 
            # the other PEB lists to spot discrepancies. 
            for base in mapped_files.keys():
                # Does the base address exist in the PEB DLL lists?
                load_mod = inloadorder.get(base, None)
                init_mod = ininitorder.get(base, None)
                mem_mod = inmemorder.get(base, None)
                # Print the full paths and base names in verbose mode 
                load = "-"
                init = "-"
                mem = "-"
                if self._config.verbose:
                    if load_mod:
                        load = "{0} : {1}".format(load_mod.FullDllName, load_mod.BaseDllName)
                    if init_mod:
                        init = "{0} : {1}".format(init_mod.FullDllName, init_mod.BaseDllName)
                    if mem_mod:
                        mem = "{0} : {1}".format(mem_mod.FullDllName, mem_mod.BaseDllName)
                    yield (0, [int(task.UniqueProcessId),
                               str(task.ImageFileName),
                               Address(base),
                               str(load_mod != None),
                               str(init_mod != None),
                               str(mem_mod != None),
                               str(mapped_files[base]),
                               str(load),
                               str(init),
                               str(mem)])
                else:
                    yield (0, [int(task.UniqueProcessId),
                               str(task.ImageFileName),
                               Address(base),
                               str(load_mod != None),
                               str(init_mod != None),
                               str(mem_mod != None),
                               str(mapped_files[base])])
                               

    def render_text(self, outfd, data):

        self.table_header(outfd,
            [("Pid", "8"),
             ("Process", "20"),
             ("Base", "[addrpad]"),
             ("InLoad", "5"),
             ("InInit", "5"),
             ("InMem", "5"),
             ("MappedPath", "")
            ])

        for task in data:
            # Build a dictionary for all three PEB lists where the
            # keys are base address and module objects are the values
            inloadorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_load_modules())
            ininitorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_init_modules())
            inmemorder = dict((mod.DllBase.v(), mod)
                                for mod in task.get_mem_modules())

            # Build a similar dictionary for the mapped files 
            mapped_files = {}
            for vad, address_space in task.get_vads(vad_filter = task._mapped_file_filter):
                # Note this is a lot faster than acquiring the full
                # vad region and then checking the first two bytes. 
                if obj.Object("_IMAGE_DOS_HEADER", offset = vad.Start, vm = address_space).e_magic != 0x5A4D:
                    continue
                mapped_files[int(vad.Start)] = str(vad.FileObject.FileName or '')

            # For each base address with a mapped file, print info on 
            # the other PEB lists to spot discrepancies. 
            for base in mapped_files.keys():
                # Does the base address exist in the PEB DLL lists?
                load_mod = inloadorder.get(base, None)
                init_mod = ininitorder.get(base, None)
                mem_mod = inmemorder.get(base, None)
                # Report if the mapped files are in the PEB lists
                self.table_row(outfd,
                        task.UniqueProcessId,
                        task.ImageFileName,
                        base,
                        str(load_mod != None),
                        str(init_mod != None),
                        str(mem_mod != None),
                        mapped_files[base]
                        )
                # Print the full paths and base names in verbose mode 
                if self._config.verbose:
                    if load_mod:
                        outfd.write("  Load Path: {0} : {1}\n".format(load_mod.FullDllName, load_mod.BaseDllName))
                    if init_mod:
                        outfd.write("  Init Path: {0} : {1}\n".format(init_mod.FullDllName, init_mod.BaseDllName))
                    if mem_mod:
                        outfd.write("  Mem Path:  {0} : {1}\n".format(mem_mod.FullDllName, mem_mod.BaseDllName))




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# Additional Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import struct
from volatility import renderers
from volatility.commands import Command
import volatility.plugins.taskmods as taskmods
import volatility.debug as debug
import volatility.obj as obj
import volatility.exceptions as exceptions
from volatility.renderers.basic import Address


class ProcDump(taskmods.DllList):
    """Dump a process to an executable file sample"""
    def __init__(self, config, *args, **kwargs):
        taskmods.DllList.__init__(self, config, *args, **kwargs)
        config.add_option('DUMP-DIR', short_option = 'D', default = None,
                          cache_invalidator = False,
                          help = 'Directory in which to dump executable files')

        config.add_option("UNSAFE", short_option = "u", default = False, action = 'store_true',
                          help = 'Bypasses certain sanity checks when creating image')
        config.add_option("MEMORY", short_option = "m", default = False, action = 'store_true',
                          help = "Carve as a memory sample rather than exe/disk")
        config.add_option('FIX', short_option = 'x', default = False,
                          help = 'Modify the image base of the dump to the in-memory base address',
                          action = 'store_true')

    def dump_pe(self, space, base, dump_file):
        """
        Dump a PE from an AS into a file.

        @param space: an AS to use
        @param base: PE base address
        @param dump_file: dumped file name

        @returns a string status message
        """

        of = open(os.path.join(self._config.DUMP_DIR, dump_file), 'wb')

        pe_file = obj.Object("_IMAGE_DOS_HEADER", offset = base, vm = space)

        try:
            for offset, code in pe_file.get_image(unsafe = self._config.UNSAFE,
                                                  memory = self._config.MEMORY,
                                                  fix = self._config.FIX):
                of.seek(offset)
                of.write(code)
            result = "OK: {0}".format(dump_file)
        except ValueError, ve:
            result = "Error: {0}".format(ve)
        except exceptions.SanityCheckException, ve:
            result = "Error: {0} Try -u/--unsafe".format(ve)
        finally:
            of.close()

        return result

    def calculate(self):
        if self._config.DUMP_DIR == None:
            debug.error("Please specify a dump directory (--dump-dir)")
        if not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        return taskmods.DllList.calculate(self)

    def unified_output(self, data):
        """Renders the tasks to disk images, outputting progress as they go"""
        return renderers.TreeGrid(
                          [("Process(V)", Address),
                           ("ImageBase", Address),
                           ("Name", str),
                           ("Result", str)],
                          self.generator(data))

    def generator(self, data):
        for task in data:
            task_space = task.get_process_address_space()
            if task_space == None:
                result = "Error: Cannot acquire process AS"
            elif task.Peb == None:
                # we must use m() here, because any other attempt to
                # reference task.Peb will try to instantiate the _PEB
                result = "Error: PEB at {0:#x} is unavailable (possibly due to paging)".format(task.m('Peb'))
            elif task_space.vtop(task.Peb.ImageBaseAddress) == None:
                result = "Error: ImageBaseAddress at {0:#x} is unavailable (possibly due to paging)".format(task.Peb.ImageBaseAddress)
            else:
                dump_file = "executable." + str(task.UniqueProcessId) + ".exe"
                result = self.dump_pe(task_space,
                                task.Peb.ImageBaseAddress,
                                dump_file)
            yield (0,
                            [Address(task.obj_offset),
                            Address(task.Peb.ImageBaseAddress),
                            str(task.ImageFileName),
                            str(result)])

    def render_text(self, outfd, data):
        """Renders the tasks to disk images, outputting progress as they go"""
        if self._config.DUMP_DIR == None:
            debug.error("Please specify a dump directory (--dump-dir)")
        if not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        self.table_header(outfd,
                          [("Process(V)", "[addrpad]"),
                           ("ImageBase", "[addrpad]"),
                           ("Name", "20"),
                           ("Result", "")])

        for task in data:
            task_space = task.get_process_address_space()
            if task_space == None:
                result = "Error: Cannot acquire process AS"
            elif task.Peb == None:
                # we must use m() here, because any other attempt to 
                # reference task.Peb will try to instantiate the _PEB
                result = "Error: PEB at {0:#x} is unavailable (possibly due to paging)".format(task.m('Peb'))
            elif task_space.vtop(task.Peb.ImageBaseAddress) == None:
                result = "Error: ImageBaseAddress at {0:#x} is unavailable (possibly due to paging)".format(task.Peb.ImageBaseAddress)
            else:
                dump_file = "executable." + str(task.UniqueProcessId) + ".exe"
                result = self.dump_pe(task_space,
                                task.Peb.ImageBaseAddress,
                                dump_file)
            self.table_row(outfd,
                            task.obj_offset,
                            task.Peb.ImageBaseAddress,
                            task.ImageFileName,
                            result)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.plthook as linux_plthook
import volatility.plugins.linux.pslist as linux_pslist
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

   
class linux_apihooks(linux_pslist.linux_pslist):
    """Checks for userland apihooks"""

    def unified_output(self, data):
        return TreeGrid([("Pid", int),
                       ("Name", str),
                       ("HookVMA", str),
                       ("HookSymbol", str),
                       ("HookedAddress", Address),
                       ("HookType", str),
                       ("HookAddress", Address),
                       ("HookLibrary", str)],
                        self.generator(data))

    def generator(self, data):
        linux_common.set_plugin_members(self)

        try:
            import distorm3
        except ImportError:
            debug.error("this plugin requres the distorm library to operate.")
         
        for task in data:
            for hook_desc, sym_name, addr, hook_type, hook_addr, hookfuncdesc in task.apihook_info():
                yield (0, [int(task.pid), str(task.comm), str(hook_desc), str(sym_name),
                        Address(addr), str(hook_type), Address(hook_addr), str(hookfuncdesc)])


    def render_text(self, outfd, data):
        self.table_header(outfd, [
              ("Pid", "7"),
              ("Name", "16"),
              ("Hook VMA", "40"),
              ("Hook Symbol", "24"),
              ("Hooked Address", "[addrpad]"),
              ("Type", "5"),
              ("Hook Address", "[addrpad]"),
              ("Hook Library", ""),
              ])

        linux_common.set_plugin_members(self)

        try:
            import distorm3
        except ImportError:
            debug.error("this plugin requres the distorm library to operate.")

        for task in data:
            for hook_desc, sym_name, addr, hook_type, hook_addr, hookfuncdesc in task.apihook_info():
                self.table_row(outfd, task.pid, task.comm, hook_desc, sym_name, addr, hook_type, hook_addr, hookfuncdesc)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.malware.malfind as malfind
import volatility.plugins.mac.pstasks as pstasks
import volatility.plugins.mac.common as common
import volatility.utils as utils 
import volatility.debug as debug
import volatility.obj as obj
import re

try:
    import yara
    has_yara = True
except ImportError:
    has_yara = False

class MapYaraScanner(malfind.BaseYaraScanner):
    """A scanner over all memory regions of a process."""

    def __init__(self, task = None, **kwargs):
        """Scan the process address space through the VMAs.

        Args:
          task: The task_struct object for this task.
        """
        self.task = task
        malfind.BaseYaraScanner.__init__(self, address_space = task.get_process_address_space(), **kwargs)

    def scan(self, offset = 0, maxlen = None, max_size = None):
        for map in self.task.get_proc_maps():
            length = map.links.end - map.links.start 
            if max_size and length > max_size:
                debug.warning("Skipping max size entry {0:#x} - {1:#x}".format(map.links.start, map.links.end))
                continue
            for match in malfind.BaseYaraScanner.scan(self, map.links.start, length):
                yield match

class mac_yarascan(malfind.YaraScan):
    """Scan memory for yara signatures"""

    def __init__(self, config, *args, **kwargs):
        malfind.YaraScan.__init__(self, config, *args, **kwargs)
        self._config.add_option('MAX-SIZE', short_option = 'M', default = 0x40000000, action = 'store', type = 'long', help = 'Set the maximum size (default is 1GB)') 

    @staticmethod
    def is_valid_profile(profile):
        return profile.metadata.get('os', 'Unknown').lower() == 'mac'

    def filter_tasks(self):
        tasks = pstasks.mac_tasks(self._config).allprocs()

        if self._config.PID is not None:        
            try:
                pidlist = [int(p) for p in self._config.PID.split(',')]
            except ValueError:
                debug.error("Invalid PID {0}".format(self._config.PID))

            pids = [t for t in tasks if t.p_pid in pidlist]
            if len(pids) == 0:
                debug.error("Cannot find PID {0}. If its terminated or unlinked, use psscan and then supply --offset=OFFSET".format(self._config.PID))
            return pids
        
        if self._config.NAME is not None:        
            try:
                name_re = re.compile(self._config.NAME, re.I)
            except re.error:
                debug.error("Invalid name {0}".format(self._config.NAME))
            
            names = [t for t in tasks if name_re.search(str(t.p_comm))]
            if len(names) == 0:
                debug.error("Cannot find name {0}. If its terminated or unlinked, use psscan and then supply --offset=OFFSET".format(self._config.NAME))
            return names

        return tasks
         
    def calculate(self):
    
        ## we need this module imported
        if not has_yara:
            debug.error("Please install Yara from https://plusvic.github.io/yara/")
            
        ## leveraged from the windows yarascan plugin
        rules = self._compile_rules()
            
        ## set the linux plugin address spaces 
        common.set_plugin_members(self)

        if self._config.KERNEL:
            ## http://fxr.watson.org/fxr/source/osfmk/mach/i386/vm_param.h?v=xnu-2050.18.24
            if self.addr_space.profile.metadata.get('memory_model', '32bit') == "32bit":
                if not common.is_64bit_capable(self.addr_space):
                    kernel_start = 0
                else:
                    kernel_start = 0xc0000000
            else:
                vm_addr = self.addr_space.profile.get_symbol("_vm_min_kernel_address")
                kernel_start = obj.Object("unsigned long", offset = vm_addr, vm = self.addr_space)

            scanner = malfind.DiscontigYaraScanner(rules = rules, 
                                                   address_space = self.addr_space) 
      
            for hit, address in scanner.scan(start_offset = kernel_start):
                yield (None, address, hit, 
                        scanner.address_space.zread(address - self._config.REVERSE, self._config.SIZE))
        else:
            # Scan each process memory block 
            tasks = self.filter_tasks()
            for task in tasks:
                # skip kernel_task
                if task.p_pid == 0:
                    continue
                scanner = MapYaraScanner(task = task, rules = rules)
                for hit, address in scanner.scan(max_size = self._config.MAX_SIZE):
                    yield (task, address, hit, 
                            scanner.address_space.zread(address - self._config.REVERSE, self._config.SIZE))
    
    def render_text(self, outfd, data):
        for task, address, hit, buf in data:
            if task:
                outfd.write("Task: {0} pid {1} rule {2} addr {3:#x}\n".format(
                    task.p_comm, task.p_pid, hit.rule, address))
            else:
                outfd.write("[kernel] rule {0} addr {1:#x}\n".format(hit.rule, address))
            
            outfd.write("".join(["{0:#018x}  {1:<48}  {2}\n".format(
                address + o, h, ''.join(c)) for o, h, c in utils.Hexdump(buf)]))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.debug as debug
import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

idt_vtype_64 = {
    'idt_desc': [ 16 , {
    'offset_low'    : [0,  ['unsigned short']],
    'segment'       : [2,  ['unsigned short']],
    'ist'           : [4,  ['unsigned short']],
    'offset_middle' : [6,  ['unsigned short']],
    'offset_high'   : [8,  ['unsigned int']],
    'unused'        : [12, ['unsigned int']],  
    }],
}

class LinuxIDTTypes(obj.ProfileModification):
    conditions = {"os" : lambda x : x in ["linux"]}

    def modification(self, profile):       
        if profile.metadata.get('memory_model', '64bit') == "64bit":
            profile.vtypes.update(idt_vtype_64)


class linux_check_idt(linux_common.AbstractLinuxCommand):
    """ Checks if the IDT has been altered """

    def calculate(self):
        """ 
        This works by walking the IDT table for the entries that Linux uses
        and verifies that each is a symbol in the kernel
        """
        linux_common.set_plugin_members(self)

        if self.profile.metadata['arch'] not in ["x64", "x86"]:
            debug.error("This plugin is only supported on Intel-based memory captures") 

        tblsz = 256

        sym_addrs = self.profile.get_all_addresses()

        # hw handlers + system call
        check_idxs = list(range(0, 20)) + [128]

        if self.profile.metadata.get('memory_model', '32bit') == "32bit":
            idt_type = "desc_struct"
        else:
            if self.profile.has_type("gate_struct64"):
                idt_type = "gate_struct64"
            else:
                idt_type = "idt_desc"

        # this is written as a list b/c there are supposdly kernels with per-CPU IDTs
        # but I haven't found one yet...
        addrs = [self.addr_space.profile.get_symbol("idt_table")]

        for tableaddr in addrs:
            table = obj.Object(theType = 'Array', offset = tableaddr, vm = self.addr_space, targetType = idt_type, count = tblsz)

            for i in check_idxs:
                ent = table[i]

                if not ent:
                    continue

                if hasattr(ent, "Address"):
                    idt_addr = ent.Address
                else:
                    low    = ent.offset_low
                    middle = ent.offset_middle
                    high   = ent.offset_high

                    idt_addr = (high << 32) | (middle << 16) | low

                if idt_addr != 0:
                    if not idt_addr in sym_addrs:
                        hooked = 1
                        sym_name = "HOOKED"
                    else:
                        hooked = 0
                        sym_name = self.profile.get_symbol_by_address("kernel", idt_addr)

                    yield(i, ent, idt_addr, sym_name, hooked)

    def unified_output(self, data):
        return TreeGrid([("Index", Address),
                       ("Address", Address),
                       ("Symbol", str)],
                        self.generator(data))

    def generator(self, data):
        for (i, _, idt_addr, sym_name, hooked) in data:
            yield (0, [Address(i), Address(idt_addr), str(sym_name)])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Index", "[addr]"), ("Address", "[addrpad]"), ("Symbol", "<30")])

        for (i, _, idt_addr, sym_name, hooked) in data:
            self.table_row(outfd, i, idt_addr, sym_name)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.list_zones as list_zones
import volatility.plugins.mac.netstat as netstat

class mac_dead_sockets(netstat.mac_netstat):
    """ Prints terminated/de-allocated network sockets """

    def calculate(self):
        common.set_plugin_members(self)
    
        zones = list_zones.mac_list_zones(self._config).calculate()

        for zone in zones:
            name = str(zone.zone_name.dereference())
            if name == "socket":
                sockets = zone.get_free_elements("socket")        
                for socket in sockets:
                    yield socket

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Proto", "6"),
                                  ("Local IP", "20"),
                                  ("Local Port", "6"),
                                  ("Remote IP", "20"),
                                  ("Remote Port", "6"),
                                  ("State", "10")])
        
        for socket in data:
            family = socket.family

            if family == 1:
                upcb = socket.so_pcb.dereference_as("unpcb")
                path = upcb.unp_addr.sun_path
                outfd.write("UNIX {0}\n".format(path))
            elif family in [2, 30]:
                proto = socket.protocol
                state = socket.state
                
                ret =  socket.get_connection_info()
               
                if ret:
                    (lip, lport, rip, rport) = ret
                else:
                    (lip, lport, rip, rport) = ("", "", "", "")

                self.table_row(outfd, proto, lip, lport, rip, rport, state)
                

  








# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import sre_constants
import struct
import volatility.plugins.procdump as procdump
import volatility.win32 as win32
import volatility.obj as obj
import volatility.utils as utils
import volatility.debug as debug
import volatility.exceptions as exceptions
from volatility.renderers import TreeGrid

class VerInfo(procdump.ProcDump):
    """Prints out the version information from PE images"""

    def __init__(self, config, *args, **kwargs):
        procdump.ProcDump.__init__(self, config, *args, **kwargs)
        config.remove_option("OFFSET")
        config.remove_option("PID")
        config.add_option("OFFSET", short_option = "o", type = 'int',
                          help = "Offset of the module to print the version information for")
        config.add_option('REGEX', short_option = "r", default = None,
                          help = 'Dump modules matching REGEX')
        config.add_option('IGNORE-CASE', short_option = 'i', action = 'store_true',
                      help = 'ignore case in pattern match', default = False)

    def calculate(self):
        """Returns a unique list of modules"""
        addr_space = utils.load_as(self._config)

        if self._config.REGEX is not None:
            try:
                if self._config.IGNORE_CASE:
                    module_pattern = re.compile(self._config.REGEX, flags = sre_constants.SRE_FLAG_IGNORECASE)
                else:
                    module_pattern = re.compile(self._config.REGEX)
            except sre_constants.error, e:
                debug.error('Regular expression parsing error: {0}'.format(e))

        if self._config.OFFSET is not None:
            if not addr_space.is_valid_address(self._config.OFFSET):
                debug.error("Specified offset is not valid for the provided address space")
            pefile = obj.Object("_IMAGE_DOS_HEADER", self._config.OFFSET, addr_space)
            if pefile.is_valid():
                yield None, pefile
            raise StopIteration

        tasks = win32.tasks.pslist(addr_space)

        for task in tasks:
            process_space = task.get_process_address_space()
            for module in task.get_load_modules():
                if self._config.REGEX is not None:
                    if not (module_pattern.search(str(module.FullDllName))
                            or module_pattern.search(str(module.BaseDllName))):
                        continue
                pefile = obj.Object("_IMAGE_DOS_HEADER", module.DllBase, process_space)
                if pefile.is_valid():
                    yield module, pefile

    def unified_output(self, data):
        return TreeGrid([("Module", str),
                       ("FileVersion", str),
                       ("ProductVersion", str),
                       ("Flags", str),
                       ("OS", str),
                       ("FileType", str),
                       ("FileDate", str), 
                       ("InfoString", str)],
                        self.generator(data))

    def generator(self, data):
        for module, pefile in data:
            if module:
                name = str(module.FullDllName)
            vinfo = pefile.get_version_info()
            if vinfo != None:
                fileversion = "{0}".format(vinfo.FileInfo.file_version())
                prodversion = "{0}".format(vinfo.FileInfo.product_version())
                flags = "{0}".format(vinfo.FileInfo.flags())
                os = "{0}".format(vinfo.FileInfo.FileOS)
                filetype = "{0}".format(vinfo.FileInfo.file_type())
                filedate = "{0}".format(vinfo.FileInfo.FileDate or '')
                infostring = ""
                for string, value in vinfo.get_file_strings():
                    infostring += "{0} : {1}".format(string, value)
                yield (0, [name,
                            fileversion,
                            prodversion,
                            flags,
                            os,
                            filetype,
                            filedate,
                            infostring])
            else:
                yield (0, [name, "", "", "", "", "", "", ""])

    def render_text(self, outfd, data):
        """Renders the text"""
        for module, pefile in data:
            if module:
                outfd.write(str(module.FullDllName))
            outfd.write("\n")
            vinfo = pefile.get_version_info()
            if vinfo != None:
                outfd.write("  File version    : {0}\n".format(vinfo.FileInfo.file_version()))
                outfd.write("  Product version : {0}\n".format(vinfo.FileInfo.product_version()))
                outfd.write("  Flags           : {0}\n".format(vinfo.FileInfo.flags()))
                outfd.write("  OS              : {0}\n".format(vinfo.FileInfo.FileOS))
                outfd.write("  File Type       : {0}\n".format(vinfo.FileInfo.file_type()))
                outfd.write("  File Date       : {0}\n".format(vinfo.FileInfo.FileDate or ''))
                for string, value in vinfo.get_file_strings():
                    outfd.write("  {0} : {1}\n".format(string, value))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.debug as debug
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_check_mig_table(common.AbstractMacCommand):
    """ Lists entires in the kernel's MIG table """

    def calculate(self):
        common.set_plugin_members(self)
       
        # we can't use an array as the size of mig_hash_entry
        # depends on if MAC_COUNTERS is set, which changes between kernels
        # mig_table_max_displ is declared directly after mig_buckets
        # which allows us to calculate the size of each entry dynamically
 
        di_addr = self.addr_space.profile.get_symbol("_mig_table_max_displ")
        mig_buckets_addr = self.addr_space.profile.get_symbol("_mig_buckets")

        ele_size = (di_addr - mig_buckets_addr) / 1024 

        for i in range(1024):
            entry = obj.Object("mig_hash_entry", offset = mig_buckets_addr + (i * ele_size), vm = self.addr_space)

            if entry.routine == 0:
                continue

            rname = self.addr_space.profile.get_symbol_by_address("kernel", entry.routine)
            if not rname or rname == "":
                rname = "HOOKED"

            yield (entry.num, rname, entry.routine)

    def unified_output(self, data):
        return TreeGrid([("Index", int),
                          ("Routine Name", str),
                          ("Routine Handler", Address),
                          ], self.generator(data))

    def generator(self, data):
        for (num, name, routine) in data:
            yield(0, [
                int(num),
                str(name),
                Address(routine),
                ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Index", "8"),
                          ("Routine Name", "100"),
                          ("Routine Handler", "[addrpad]")])

        for (num, name, routine) in data:
            self.table_row(outfd, num, name, routine) 




# Volatility
# Copyright (c) 2008-2015 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
import volatility.utils as utils
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.common as common
import volatility.plugins.malware.devicetree as dtree
import volatility.win32.modules as modules
import volatility.win32.tasks as tasks
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class drivermodule(common.AbstractWindowsCommand):
    """Associate driver objects to kernel modules"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option('ADDR', short_option = 'a', default = None,
                          help = 'Show info on module at or containing this address',
                          action = 'store', type = 'int')
 
    def calculate(self):
        addr_space = utils.load_as(self._config)

        modlist = list(modules.lsmod(addr_space))
        mods = dict((addr_space.address_mask(mod.DllBase), mod) for mod in modlist)
        mod_addrs = sorted(mods.keys())
            
        drivers = dtree.DriverIrp(self._config).calculate()    
        found_driver = "UNKNOWN"

        if self._config.ADDR:
            find_address = self._config.ADDR
            
            found_module = tasks.find_module(mods, mod_addrs, mods.values()[0].obj_vm.address_mask(find_address))
            if found_module:
                found_module = found_module.BaseDllName or found_module.FullDllName
            else:
                found_module = "UNKNOWN"

            for driver in drivers:
                if driver.DriverStart <= find_address < driver.DriverStart + driver.DriverSize:
                    header = driver.get_object_header()
                    found_driver = header.NameInfo.Name
                    break
            
            yield (found_module, found_driver)

        else:                
            for driver in drivers:
                driver_name = driver.get_object_header().NameInfo.Name
                owning_module = tasks.find_module(mods, mod_addrs, mods.values()[0].obj_vm.address_mask(driver.DriverStart))

                if owning_module:
                    module_name = owning_module.BaseDllName or owning_module.FullDllName
                else:
                    module_name = "UNKNOWN"

                yield (module_name, driver_name)

    def generator(self, data):

        for module_name, driver_name in data:
            yield( 0, [str(module_name), str(driver_name)])

    def unified_output(self, data):
        return TreeGrid([("Module", str),
                         ("Driver", str)],
                        self.generator(data))

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Module", "36"), ("Driver", "")])

        for module_name, driver_name in data:
            self.table_row(outfd, module_name, driver_name)
       






# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj

#----------------------------------------------------------------------
# All Windows
#----------------------------------------------------------------------

class VadTraverser(obj.CType):

    ## The actual type depends on this tag value.
    tag_map = {'Vadl': '_MMVAD_LONG',
               'VadS': '_MMVAD_SHORT',
               'Vad ': '_MMVAD_LONG',
               'VadF': '_MMVAD_SHORT',
               'Vadm': '_MMVAD_LONG',
              }

    def is_valid(self):
        return (obj.CType.is_valid(self) and
                self.Start < obj.VolMagic(self.obj_vm).MaxAddress.v() and
                self.End < (obj.VolMagic(self.obj_vm).MaxAddress.v()))

    def traverse(self, visited = None, depth = 0):
        """ Traverse the VAD tree by generating all the left items,
        then the right items.

        We try to be tolerant of cycles by storing all offsets visited.
        """

        if depth > 100:
            raise RuntimeError("Vad tree too deep - something went wrong!")

        if visited == None:
            visited = set()

        ## We try to prevent loops here
        if self.obj_offset in visited:
            return

        # Find out which Vad type we need to be:
        if str(self.Tag) in self.tag_map:
            yield self.cast(self.tag_map[str(self.Tag)])

        # This tag is valid for the Root.
        elif depth and str(self.Tag) != "":
            return

        # add this node to those that have been visited
        visited.add(self.obj_offset)
 
        # traverse children
        for c in self.LeftChild.traverse(visited = visited, depth = depth + 1):
            yield c

        for c in self.RightChild.traverse(visited = visited, depth = depth + 1):
            yield c

class VadFlags(obj.CType):
    def __str__(self):
        return ", ".join(["{0}: {1}".format(name, self.m(name)) for name in sorted(self.members.keys()) if self.m(name) != 0])

class _MMVAD_FLAGS(VadFlags):
    pass

class _MMVAD_FLAGS2(VadFlags):
    pass

class _MMSECTION_FLAGS(VadFlags):
    pass

class VadFlagsModification(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x : x == "windows"}

    def modification(self, profile):
        profile.object_classes.update({
            '_MMVAD_FLAGS': _MMVAD_FLAGS,
            '_MMVAD_FLAGS2': _MMVAD_FLAGS2,
            '_MMSECTION_FLAGS': _MMSECTION_FLAGS,
            })

class VadTagModification(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x : x == "windows"}

    def modification(self, profile):

        version = (profile.metadata.get("major", 0), 
                   profile.metadata.get("minor", 0))

        model = profile.metadata.get("memory_model", "32bit")

        if model == "32bit":
            offset = -4
        else:
            offset = -12

        overlay = {
            '_MMVAD_SHORT': [ None, {
                'Tag': [offset , ['String', dict(length = 4)]],
            }],
            '_MMVAD': [ None, { 
                'Tag': [offset , ['String', dict(length = 4)]],
            }]}

        if version < (6, 2):
            overlay.update({
                '_MMVAD_LONG': [ None, {
                    'Tag': [offset , ['String', dict(length = 4)]],
                }]})

        if version >= (5, 2) and version <= (6, 1):
            overlay.update({
                '_MMADDRESS_NODE': [ None, {
                    'Tag': [offset , ['String', dict(length = 4)]],
                }]})
        elif version == (6, 2):
            overlay.update({
                '_MM_AVL_NODE': [ None, {
                    'Tag': [offset , ['String', dict(length = 4)]],
                }]})
        elif version >= (6, 3):
            overlay.update({
                '_RTL_BALANCED_NODE': [ None, {
                    'Tag': [offset , ['String', dict(length = 4)]],
                }]})

        profile.merge_overlay(overlay)

#----------------------------------------------------------------------
# Windows XP 
#----------------------------------------------------------------------

class _MMVAD_SHORT_XP(VadTraverser):

    @property
    def Parent(self):
        return self.m('Parent').dereference()

    @property
    def Start(self):
        return self.StartingVpn << 12

    @property
    def End(self):
        return ((self.EndingVpn + 1) << 12) - 1

    @property
    def Length(self):
        return ((self.EndingVpn + 1) << 12) - self.Start 

    @property
    def VadFlags(self):
        return self.u.VadFlags

    @property
    def CommitCharge(self):
        return self.u.VadFlags.CommitCharge

class _MMVAD_XP(_MMVAD_SHORT_XP):

    @property
    def ControlArea(self):
        return self.m('ControlArea')

    @property
    def FileObject(self):
        return self.ControlArea.FilePointer.dereference()

class _MMVAD_LONG_XP(_MMVAD_XP):
    pass

class WinXPx86Vad(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x: x == "windows", 
                  "major": lambda x: x == 5, 
                  "minor": lambda x: x == 1, 
                  "memory_model": lambda x: x == "32bit"}

    def modification(self, profile):
        profile.merge_overlay({
            '_EPROCESS' : [ None, {
                'VadRoot' : [ None, ['pointer', ['_MMVAD']]],
            }]})

        profile.object_classes.update({
            '_MMVAD': _MMVAD_XP,
            '_MMVAD_SHORT': _MMVAD_SHORT_XP,
            '_MMVAD_LONG': _MMVAD_LONG_XP,
            })

#----------------------------------------------------------------------
# Windows 2003 
#----------------------------------------------------------------------

class _MMVAD_SHORT_2003(_MMVAD_SHORT_XP):

    @property
    def Parent(self):
        return obj.Object("_MMADDRESS_NODE", 
                    vm = self.obj_vm, 
                    offset = self.u1.Parent.v() & ~0x3, 
                    parent = self.obj_parent)

class _MMVAD_2003(_MMVAD_SHORT_2003):
    
    @property
    def ControlArea(self):
        return self.m('ControlArea')

    @property
    def FileObject(self):
        return self.ControlArea.FilePointer.dereference()

class _MMVAD_LONG_2003(_MMVAD_2003):
    pass

class _MM_AVL_TABLE(obj.CType):
    def traverse(self):
        for c in self.cast("_MMADDRESS_NODE").traverse():
            yield c

class Win2003x86Vad(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x: x == "windows", 
                  "major": lambda x: x == 5, 
                  "minor": lambda x: x == 2}

    def modification(self, profile):
        profile.object_classes.update({
            '_MMVAD': _MMVAD_2003,
            '_MMVAD_SHORT': _MMVAD_SHORT_2003,
            '_MMVAD_LONG': _MMVAD_LONG_2003,
            '_MM_AVL_TABLE': _MM_AVL_TABLE,
            '_MMADDRESS_NODE': _MMVAD_2003,
            })

#----------------------------------------------------------------------
# Windows Vista, 2008, and 7
#----------------------------------------------------------------------

class _MMVAD_VISTA(_MMVAD_SHORT_2003):
    
    @property
    def ControlArea(self):
        return self.Subsection.ControlArea

    @property
    def FileObject(self):
        return self.Subsection.ControlArea.FilePointer.dereference_as("_FILE_OBJECT")

class _MMVAD_LONG_VISTA(_MMVAD_VISTA):
    pass

class VistaVad(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x: x == "windows", 
                  "major": lambda x: x == 6, 
                  "minor": lambda x: (x == 0 or x == 1)}

    def modification(self, profile):
        profile.object_classes.update({
            '_MMVAD': _MMVAD_VISTA,
            '_MMVAD_SHORT': _MMVAD_SHORT_2003,
            '_MMVAD_LONG': _MMVAD_LONG_VISTA,
            '_MM_AVL_TABLE': _MM_AVL_TABLE,
            '_MMADDRESS_NODE': _MMVAD_VISTA,
            })

#----------------------------------------------------------------------
# Windows 8 and Server 2012
#----------------------------------------------------------------------

class _MM_AVL_TABLE_WIN8(obj.CType):
    def traverse(self):
        for c in self.cast("_MM_AVL_NODE").traverse():
            yield c

class _MM_AVL_NODE(VadTraverser):

    ## The actual type depends on this tag value.
    tag_map = {'Vadl': '_MMVAD',
               'VadS': '_MMVAD_SHORT',
               'Vad ': '_MMVAD',
               'VadF': '_MMVAD_SHORT',
               'Vadm': '_MMVAD',
              }

class _MMVAD_SHORT_WIN8(_MM_AVL_NODE):

    @property
    def Parent(self):
        return obj.Object("_MM_AVL_NODE", 
                    vm = self.obj_vm, 
                    offset = self.VadNode.u1.Parent.v() & ~0x3, 
                    parent = self.obj_parent)

    @property
    def Start(self):
        return self.StartingVpn << 12

    @property
    def End(self):
        return ((self.EndingVpn + 1) << 12) - 1

    @property
    def VadFlags(self):
        return self.u.VadFlags

    @property
    def CommitCharge(self):
        return self.u1.VadFlags1.CommitCharge

    @property
    def Length(self):
        return self.End - self.Start 

    @property
    def LeftChild(self):
        return self.VadNode.LeftChild

    @property
    def RightChild(self):
        return self.VadNode.RightChild

class _MMVAD_WIN8(_MM_AVL_NODE):

    @property
    def Parent(self):
        return self.Core.Parent

    @property
    def Start(self):
        return self.Core.Start
    
    @property
    def End(self):
        return self.Core.End

    @property
    def VadFlags(self):
        return self.Core.VadFlags
    
    @property
    def CommitCharge(self):
        return self.Core.CommitCharge

    @property
    def ControlArea(self):
        return self.Subsection.ControlArea

    @property
    def FileObject(self):
        return self.Subsection.ControlArea.FilePointer.dereference_as("_FILE_OBJECT")

    @property
    def Length(self):
        return self.End - self.Start 

    @property
    def LeftChild(self):
        return self.Core.LeftChild

    @property
    def RightChild(self):
        return self.Core.RightChild

class Win8Vad(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x: x == "windows", 
                  "major": lambda x: x == 6, 
                  "minor": lambda x: x == 2}

    def modification(self, profile):
        profile.object_classes.update({
            '_MMVAD': _MMVAD_WIN8,
            '_MMVAD_SHORT': _MMVAD_SHORT_WIN8,
            '_MM_AVL_TABLE': _MM_AVL_TABLE_WIN8,
            '_MM_AVL_NODE': _MM_AVL_NODE,
            })

#----------------------------------------------------------------------
# Windows 8.1 and Server 2012 R2
#----------------------------------------------------------------------

class _RTL_AVL_TREE(obj.CType):
    def traverse(self):
        for x in self.Root.traverse():
            yield x

class _RTL_BALANCED_NODE(VadTraverser):

    ## The actual type depends on this tag value.
    tag_map = {'Vadl': '_MMVAD',
               'VadS': '_MMVAD_SHORT',
               'Vad ': '_MMVAD',
               'VadF': '_MMVAD_SHORT',
               'Vadm': '_MMVAD',
              }

    @property
    def LeftChild(self):
        return self.Left

    @property 
    def RightChild(self):
        return self.Right

class _MMVAD_SHORT_WIN81(_RTL_BALANCED_NODE):

    @property
    def Parent(self):
        return obj.Object("_RTL_BALANCED_NODE", 
                    vm = self.obj_vm, 
                    offset = self.VadNode.ParentValue.v() & ~0x3, 
                    parent = self.obj_parent)

    @property
    def Start(self):
        return (self.StartingVpn << 12) | (self.StartingVpnHigh << 44)

    @property
    def End(self):
        return (((self.EndingVpn + 1) << 12) | (self.EndingVpnHigh << 44)) - 1

    @property
    def VadFlags(self):
        return self.u.VadFlags

    @property
    def CommitCharge(self):
        return self.u1.VadFlags1.CommitCharge

    @property
    def Length(self):
        return self.End - self.Start 

    @property
    def LeftChild(self):
        return self.VadNode.Left

    @property
    def RightChild(self):
        return self.VadNode.Right

class _MMVAD_WIN81(_MMVAD_SHORT_WIN81):

    @property
    def Parent(self):
        return self.Core.Parent

    @property
    def Start(self):
        return self.Core.Start
    
    @property
    def End(self):
        return self.Core.End

    @property
    def VadFlags(self):
        return self.Core.VadFlags
    
    @property
    def CommitCharge(self):
        return self.Core.CommitCharge

    @property
    def ControlArea(self):
        return self.Subsection.ControlArea

    @property
    def FileObject(self):
        return self.Subsection.ControlArea.FilePointer.dereference_as("_FILE_OBJECT")

    @property
    def Length(self):
        return self.End - self.Start 

    @property
    def LeftChild(self):
        return self.Core.LeftChild

    @property
    def RightChild(self):
        return self.Core.RightChild

class Win81Vad(obj.ProfileModification):

    before = ["WindowsOverlay"]
    conditions = {"os": lambda x: x == "windows", 
                  "major": lambda x: x == 6, 
                  "minor": lambda x: x >= 3}

    def modification(self, profile):
        profile.object_classes.update({
            '_MMVAD': _MMVAD_WIN81,
            '_MMVAD_SHORT': _MMVAD_SHORT_WIN81,
            '_RTL_AVL_TREE': _RTL_AVL_TREE,
            '_RTL_BALANCED_NODE': _RTL_BALANCED_NODE,
            })




# Volatility
# Copyright (C) 2014 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

""" An AS for processing Windows Bitmap crash dumps """
import struct
import volatility.obj as obj
import volatility.addrspace as addrspace
import volatility.plugins.addrspaces.crash as crash

#pylint: disable-msg=C0111

class BitmapDmpVTypes(obj.ProfileModification):

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == "64bit"}

    def modification(self, profile):
        profile.vtypes.update({
              '_FULL_DUMP64' : [ 0x38, {
                'Signature' : [ 0x0, ['array', 4, ['unsigned char']]],
                'ValidDump' : [ 0x4, ['array', 4, ['unsigned char']]],
                'DumpOptions' : [ 0x8, ['unsigned long long']],
                'HeaderSize' : [ 0x20, ['unsigned long long']],  
                'BitmapSize' : [ 0x28, ['unsigned long long']],  
                'Pages' : [ 0x30, ['unsigned long long']], 
                'Buffer' : [ 0x38, ['array', lambda x: (x.BitmapSize+7) / 0x8, ['unsigned char']]],
                'Buffer2' : [ 0x38, ['array', lambda x: (x.BitmapSize + 31) / 32, ['unsigned long']]],
            } ],
            })


class WindowsCrashDumpSpace64BitMap(crash.WindowsCrashDumpSpace32):
    """ This AS supports Windows BitMap Crash Dump format """
    order = 29
    dumpsig = 'PAGEDU64'
    headertype = "_DMP_HEADER64"
    headerpages = 0x13
    bitmaphdroffset = 0x2000 

    def __init__(self, base, config, **kwargs):
        ## We must have an AS below us
        self.as_assert(base, "No base Address Space")

        addrspace.AbstractRunBasedMemory.__init__(self, base, config, **kwargs)

        ## Must start with the magic PAGEDUMP
        self.as_assert((base.read(0, 8) == self.dumpsig), "Header signature invalid")

        self.as_assert(self.profile.has_type(self.headertype), self.headertype + " not available in profile")
        self.header = obj.Object(self.headertype, 0, base)

        # This address space supports Windows Bitmap crash dump files
        # which, based on empirical tests, have a DumpType of 0x5.  
        self.as_assert((self.header.DumpType == 5), "Unsupported dump format")

        # Instantiate the Summary/Full Bitmap header
        self.bitmaphdr = obj.Object("_FULL_DUMP64", self.bitmaphdroffset, base)

        # Create a cached version of the Header/Bitmap to reduce I/O
        fdmp_buff = base.read(self.bitmaphdroffset, self.bitmaphdr.HeaderSize-self.bitmaphdroffset)
        bufferas = addrspace.BufferAddressSpace(self._config, data = fdmp_buff)
        self.bitmaphdr2 = obj.Object('_FULL_DUMP64', vm = bufferas, offset = 0)

        firstbit = None                         # First bit in a run
        firstoffset = 0                         # File offset of first bit 
        lastbit = None                          # Last bit in a run
        lastbitseen = 0                         # Most recent bit processed
        offset = self.bitmaphdr2.HeaderSize     # Size of file headers

        for i in range(0, ((self.bitmaphdr2.BitmapSize + 31) / 32)):
            if self.bitmaphdr.Buffer2[i] == 0:
                 if firstbit != None:
                    lastbit = ((i - 1) * 32) + 31
                    self.runs.append((firstbit * 0x1000, firstoffset, (lastbit - firstbit + 1) * 0x1000))
                    firstbit = None
            elif self.bitmaphdr.Buffer2[i] == 0xFFFFFFFF:
                 if firstbit == None:
                     firstoffset = offset
                     firstbit = i * 32
                 offset = offset + (32 * 0x1000)
            else:
                 wordoffset = i * 32
                 for j in range(0, 32):
                     BitAddr = wordoffset + j 
                     ByteOffset = BitAddr >> 3
                     ByteAddress = (self.bitmaphdr2.Buffer[ByteOffset])
                     ShiftCount = (BitAddr & 0x7)
                     if ((ByteAddress >> ShiftCount) & 1):
                         if firstbit == None:
                             firstoffset = offset
                             firstbit = BitAddr
                         offset = offset + 0x1000
                     else:
                         if firstbit != None:
                             lastbit = BitAddr - 1
                             self.runs.append((firstbit * 0x1000, firstoffset, (lastbit - firstbit + 1) * 0x1000))
                             firstbit = None
            lastbitseen = (i * 32) + 31

        if firstbit != None:
            self.runs.append((firstbit * 0x1000, firstoffset, (lastbitseen - firstbit + 1) * 0x1000))

        self.dtb = self.header.DirectoryTableBase.v()



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2014 CrowdStrike, Inc.
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Georg Wicherski
@license:      GNU General Public License 2.0
@contact:      georg@crowdstrike.com
@organization: CrowdStrike, Inc.
"""

import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as linux_pslist
import volatility.plugins.linux.dump_map as linux_dump_map
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_elfs(linux_pslist.linux_pslist):
    """Find ELF binaries in process mappings"""

    def calculate(self):
        linux_common.set_plugin_members(self)
        tasks = linux_pslist.linux_pslist.calculate(self)

        for task in tasks:
            for elf, elf_start, elf_end, soname, needed in task.elfs():
                yield task, elf, elf_start, elf_end, soname, needed
    
    def unified_output(self, data):
        return TreeGrid([("Pid", int),
                       ("Name", str),
                       ("Start", Address),
                       ("End", Address),
                       ("Path", str),
                       ("Needed", str)],
                        self.generator(data))

    def generator(self, data):
        for task, elf, start, end, soname, needed in data:
            yield (0, [int(task.pid), str(task.comm), Address(start), Address(end), str(soname), ",".join(needed)])
 

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Pid", "8"),
                                  ("Name", "17"),
                                  ("Start", "[addrpad]"),
                                  ("End", "[addrpad]"),
                                  ("Elf Path", "60"),
                                  ("Needed", "")
                                ])
        for task, elf, start, end, soname, needed in data:
            self.table_row(outfd, task.pid, task.comm, start, end, soname, ",".join(needed))




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.mac.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_lsmod_iokit(common.AbstractMacCommand):
    """ Lists loaded kernel modules through IOkit """

    def _struct_or_class(self, type_name):
        """Return the name of a structure or class. 

        More recent versions of OSX define some types as 
        classes instead of structures, so the naming is
        a little different.   
        """
        if self.addr_space.profile.vtypes.has_key(type_name):
            return type_name
        else:
            return type_name + "_class"


    def calculate(self):
        common.set_plugin_members(self)

        saddr  = common.get_cpp_sym("sLoadedKexts", self.addr_space.profile)

        p = obj.Object("Pointer", offset = saddr, vm = self.addr_space) 

        kOSArr = obj.Object(self._struct_or_class("OSArray"), offset = p, vm = self.addr_space)

        if kOSArr == None:
            debug.error("The OSArray_class type was not found in the profile. Please file a bug if you are running aginst Mac >= 10.7")

        kext_arr = obj.Object(theType  = "Array", targetType = "Pointer", offset = kOSArr.array, count = kOSArr.capacity, vm = self.addr_space)

        for (i, kext) in enumerate(kext_arr):
            kext = kext.dereference_as(self._struct_or_class("OSKext"))
            if kext and kext.is_valid():
                yield kext

    def unified_output(self, data):
        return TreeGrid([("Offset (V)", Address),
                                  ("Module Address", Address),
                                  ("Size", str),
                                  ("Refs", str),
                                  ("Version", str),
                                  ("Name", str),
                                  ("Path", str)
                                  ], self.generator(data))

    def generator(self, data):
        for kext in data:
            path = kext.path

            if path:
                path = str(path.dereference())

            yield(0, [
                      Address(kext.kmod_info),
                      Address(kext.kmod_info.address),
                      str(kext.kmod_info.m("size")),
                      str(kext.kmod_info.reference_count),
                      str(kext.version),
                      str(kext.kmod_info.name),
                      str(path)
                      ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"),
                                  ("Module Address", "[addrpad]"), 
                                  ("Size", "8"), 
                                  ("Refs", "^8"),
                                  ("Version", "12"),  
                                  ("Name", "48"),
                                  ("Path", "")])
        for kext in data:
            path = kext.path

            if path:
                path = str(path.dereference())

            self.table_row(outfd,
                           kext.kmod_info,
                           kext.kmod_info.address, 
                           kext.kmod_info.m("size"),
                           kext.kmod_info.reference_count, 
                           kext.version,
                           kext.kmod_info.name, 
                           str(path))




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import re, os, struct
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common

class linux_lsmod(linux_common.AbstractLinuxCommand):
    """Gather loaded kernel modules"""

    def __init__(self, config, *args, **kwargs):

        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
      
        self._config.add_option('SECTIONS', short_option = 'T', default = None, help = 'show section addresses', action = 'store_true')
        self._config.add_option('PARAMS', short_option = 'P', default = None, help = 'show module parameters', action = 'store_true')
        self._config.add_option('BASE', short_option = 'b', default = None, help = 'Dump driver with BASE address (in hex)', action = 'store', type = 'int')
        self._config.add_option('IDC', short_option = 'c', default = None, help = 'Path to IDC file to be created for module', action = 'store', type = 'str')
        

    def _get_modules(self):
        if self._config.BASE:
            module_address = int(self._config.BASE)
            yield obj.Object("module", offset = module_address, vm = self.addr_space)
        else:
            modules_addr = self.addr_space.profile.get_symbol("modules")

            modules = obj.Object("list_head", vm = self.addr_space, offset = modules_addr)

            # walk the modules list
            for module in modules.list_of_type("module", "list"):
                yield module

    def calculate(self):
        linux_common.set_plugin_members(self)
        
        for module in self._get_modules():
            if self._config.PARAMS:
                if not hasattr(module, "kp"):
                    debug.error("Gathering module parameters is not supported in this profile.")

                params = module.get_params()
            else:
                params = ""

            if self._config.SECTIONS:
                sections = module.get_sections()
            else:
                sections = []

            yield (module, sections, params)

    def render_text(self, outfd, data):

        for (module, sections, params)  in data:
            if self._config.IDC:
                fd = open(self._config.IDC, "w")
                fd.write("#include <idc.idc>\nstatic main(void) {\n")
                
                for (sname, saddr) in module.get_symbols():             
                    fd.write("   MakeDword(0x{0:08X});\n".format(saddr))
                    fd.write("   MakeName(0x{0:08X}, \"{1}\");\n".format(saddr, sname))

                fd.write("}")

            outfd.write("{2:x} {0:s} {1:d}\n".format(module.name, module.init_size + module.core_size, module.obj_offset))

            # will be empty list if not set on command line
            for sect in sections:
                outfd.write("\t{0:30s} {1:#x}\n".format(sect.sect_name, sect.address))

            # will be "" if not set, otherwise will be space seperated
            if params != "":
                for param in params.split():
                    outfd.write("\t{0:100s}\n".format(param))

    def get_module(self, name):
        ret = None

        for (module, _, _) in self.calculate():
            if str(module.name) == name:
                ret = module
                break

        return ret

    # returns a list of tuples of (name, .text start, .text end) for each module
    # include_list can contain a list of only the modules wanted by a plugin
    def get_modules(self, include_list = None):
        if not include_list:
            include_list = []

        ret = []
        for (module, _sections, _params) in self.calculate():

            if len(include_list) == 0 or str(module.name) in include_list:

                start = module.module_core
                end = start + module.core_size
                ret.append(("%s" % module.name, start, end))

        return ret

class linux_moddump(linux_common.AbstractLinuxCommand):
    """Extract loaded kernel modules"""
    
    def __init__(self, config, *args, **kwargs):
        self.name_idx = 1
        self.idc_started = False

        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
        
        config.add_option('DUMP-DIR', short_option = 'D', default = None,       
                      help = 'Directory in which to dump the files',
                      action = 'store', type = 'string')
        config.add_option('REGEX', short_option = 'r',
                      help = 'Dump modules matching REGEX',
                      action = 'store', type = 'string')
        config.add_option('IGNORE-CASE', short_option = 'i',
                      help = 'Ignore case in pattern match',
                      action = 'store_true', default = False)
        config.add_option('BASE', short_option = 'b', default = None,
                          help = 'Dump driver with BASE address (in hex)',
                          action = 'store', type = 'int')

    def calculate(self):
        linux_common.set_plugin_members(self)

        if self._config.REGEX:
            try:
                if self._config.IGNORE_CASE:
                    mod_re = re.compile(self._config.REGEX, re.I)
                else:
                    mod_re = re.compile(self._config.REGEX)
            except re.error, e:
                debug.error('Error parsing regular expression: {0}'.format(e))
                

        if self._config.BASE:
            module_address = int(self._config.BASE)
            yield obj.Object("module", offset = module_address, vm = self.addr_space)
        else:
            # walk the modules list
            modules_addr = self.addr_space.profile.get_symbol("modules")
            modules = obj.Object("list_head", vm = self.addr_space, offset = modules_addr)
            for module in modules.list_of_type("module", "list"):
                if self._config.REGEX:
                    if not mod_re.search(str(module.name)):
                        continue
                yield module
    
    def _get_header_64(self, load_addr, sect_hdr_offset, num_sects):
        e_ident     = "\x7f\x45\x4c\x46\x02\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        e_type      = "\x01\x00" # relocateble
        e_machine   = "\x03\x00"
        e_version   = "\x01\x00\x00\x00"
        e_entry     = "\x00" * 8
        e_phoff     = "\x00" * 8
        e_shoff     = struct.pack("<Q", sect_hdr_offset)
        e_flags     = "\x00\x00\x00\x00"
        e_ehsize    = "\x40\x00"
        e_phentsize = "\x00\x00"
        e_phnum     = "\x00\x00"
        e_shentsize = "\x40\x00"
        e_shnum     = struct.pack("<H", num_sects + 1) # this works as we stick the seciton we create at the end
        e_shstrndx  = struct.pack("<H", num_sects)

        header = e_ident + e_type + e_machine + e_version + e_entry + e_phoff + e_shoff + e_flags
    
        header = header + e_ehsize + e_phentsize + e_phnum + e_shentsize + e_shnum + e_shstrndx

        if len(header) != 64:
            debug.error("BUG: ELF header not bytes. %d" % len(header))

        return header

    def _get_header_32(self, load_addr, sect_hdr_offset, num_sects):
        e_ident     = "\x7f\x45\x4c\x46\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        e_type      = "\x01\x00" # relocateble
        e_machine   = "\x03\x00"
        e_version   = "\x01\x00\x00\x00"
        e_entry     = "\x00" * 4
        e_phoff     = "\x00" * 4
        e_shoff     = struct.pack("<I", sect_hdr_offset)
        e_flags     = "\x00\x00\x00\x00"
        e_ehsize    = "\x34\x00"
        e_phentsize = "\x00\x00"
        e_phnum     = "\x00\x00"
        e_shentsize = "\x28\x00"
        e_shnum     = struct.pack("<H", num_sects + 1) # this works as we stick the seciton we create at the end
        e_shstrndx  = struct.pack("<H", num_sects)

        header = e_ident + e_type + e_machine + e_version + e_entry + e_phoff + e_shoff + e_flags
    
        header = header + e_ehsize + e_phentsize + e_phnum + e_shentsize + e_shnum + e_shstrndx

        if len(header) != 52:
            debug.error("BUG: ELF header not bytes. %d" % len(header))

        return header

    # checked
    def _build_sections_list(self, module):
        sections = []
        
        symtab_idx = -1

        for (i, sect) in enumerate(module.get_sections()):
            name = str(sect.sect_name)

            sections.append((name, sect.address.v()))
                         
            if name == ".symtab":
                symtab_idx = i

        if symtab_idx == -1:
            debug.error("No section .symtab found. Unable to properly re-create ELF file.")

        return (sections, symtab_idx)

    # we do this in a couple phases:
    # 1) walk the volatlity get_sections
    # 2) this gives us the name and start address of each
    # 3) we use the list we build in build_.. so then we can calcluate the size of each
    # 4) with the final list of name,address,size we can read the sections and populate the info in the file
    def _parse_sections(self, module):
        (orig_sections, symtab_idx) = self._build_sections_list(module)
       
        if self.addr_space.profile.metadata.get('memory_model', '32bit') == '64bit':
            sect_bytes = 64
        else:
            sect_bytes = 52

        updated_sections = []
        tmp_ents = {}

        for (_i, (name, address)) in enumerate(orig_sections):
            tmp_ents[address] = 1

        addrs = sorted(tmp_ents.keys())
       
        sorted_ents = {}
        for (i, addr) in enumerate(addrs):
            sorted_ents[addr] = i
        
        sect_sa = []
        # do this twice for now... just want the plugin working!
        for (i, (name, address)) in enumerate(orig_sections):
            sort_idx = sorted_ents[address]

            if name == ".symtab":
                print "FIXING"
                str_section_data = self._fix_sym_table(module, sect_sa)
                str_size = len(str_section_data)
                size = str_size
            else:
                try:
                    next_addr = addrs[sort_idx+1]
                    size = next_addr - address
                    #print "%x | %x | %8x | %d | %s" % (next_addr, address, size, sort_idx, name)
                except IndexError:
                    # the last one
                    size = 0x4000 # guess?
                    #print "one: size 0x4000 for %s" % name

            sect_sa.append((name, address, size))

        for (i, (name, address)) in enumerate(orig_sections):
            sort_idx = sorted_ents[address]

            try:
                next_addr = addrs[sort_idx+1]
                size = next_addr - address
                #print "%x | %x | %8x | %d | %s" % (next_addr, address, size, sort_idx, name)
            except IndexError:
                # the last one
                size = 0x4000 # guess?

            if name == ".symtab":
                print "FIXING 2"
                size         = str_size
                section_data = str_section_data
            else:  
                section_data = module.obj_vm.zread(address, size)
                #print "reading %d bytes from %x for section %s : got %d" % (size, address, name, len(section_data))

            print "adding section %s | %d with size %d | %d" % (name, sect_bytes, size, len(section_data))
            
            updated_sections.append((name, address, size, sect_bytes, section_data))
            
            sect_bytes = sect_bytes + size
       
        return (updated_sections, symtab_idx, addrs[0])

    def _calc_sect_name_idx(self, name):
        ret = self.name_idx
        self.name_idx = self.name_idx + len(name) + 1
        return ret

    def _calc_sect_type(self, name):
        type_map = {
            "SHT_NULL" : 0,
            "SHT_PROGBITS" : 1,
            "SHT_SYMTAB" : 2,
            "SHT_STRTAB" : 3,
            "SHT_RELA" : 4,
            "SHT_HASH" : 5,
            "SHT_DYNAMIC" : 6,
            "SHT_NOTE" : 7,
            "SHT_NOBITS" : 8,
            "SHT_REL" : 9,
            "SHT_SHLIB" : 10,
            "SHT_DYNSYM" : 11,
            "SHT_LOPROC" : 0x70000000,
            "SHT_HIPROC" : 0x7fffffff,
            "SHT_LOUSER" : 0x80000000,
            "SHT_HIUSER" : 0xffffffff
        }
         
        known_sections = {
            ".note.gnu.build-id" : "SHT_NOTE",
            ".text"              : "SHT_PROGBITS",
            ".rodata"            : "SHT_PROGBITS",
            ".modinfo"           : "SHT_PROGBITS",
            "__param"            : "SHT_PROGBITS",
            ".data"              : "SHT_PROGBITS",
            ".gnu.linkonce.this_module" : "SHT_PROGBITS",
            ".comment"                  : "SHT_PROGBITS",
            ".shstrtab"                 : "SHT_STRTAB",
            ".symtab"                   : "SHT_SYMTAB",
            ".strtab"                   : "SHT_STRTAB",
            } 

        if name in known_sections:
            sect_type_name = known_sections[name]
            sect_type_val  = type_map[sect_type_name]
        else:
            sect_type_val = 1 # SHT_PROGBITS
         
        if name.find(".rela.") != -1:
            sect_type_val = 4 # SHT_RELA

        return sect_type_val

    # all sections from memory are allocated (SHF_ALLOC)
    # special check certain other sections to try and ensure extra flags are added where needed
    def _calc_sect_flags(self, name):
        flags = 2 # SHF_ALLOC
        
        if name == ".text":
            flags = flags | 4 # SHF_EXECINSTR
        
        elif name in [".data", ".bss"]:
            flags = flags | 1 # SHF_WRITE

        return flags

    def _calc_link(self, name, strtab_idx, symtab_idx, sect_type):
        # looking for RELA sections
        if name.find(".rela.") != -1: 
            lnk = strtab_idx

        elif sect_type == 2: # strtab
            lnk = strtab_idx

        else:
            lnk = 0

        return lnk

    def _calc_entsize(self, name, sect_type, bits):
        # looking for RELA sections
        if name.find(".rela.") != -1: 
            info = 24

        elif sect_type == 2: # symtab
            if bits == 32:
                info = 16
            else:
                info = 24
        else:
            info = 0

        return info
    
    def _make_sect_header_64(self, name, address, size, file_off, strtab_idx, symtab_idx):
        int_sh_type = self._calc_sect_type(name)

        sh_name       = struct.pack("<I", self._calc_sect_name_idx(name))
        sh_type       = struct.pack("<I", int_sh_type)
        sh_flags      = struct.pack("<Q", self._calc_sect_flags(name))
        sh_addr       = struct.pack("<Q", address)
        sh_offset     = struct.pack("<Q", file_off)
        sh_size       = struct.pack("<Q", size)
        sh_link       = struct.pack("<I", self._calc_link(name, strtab_idx, symtab_idx, int_sh_type))
        sh_info       = "\x00" * 4 
        sh_addralign  = "\x01\x00\x00\x00\x00\x00\x00\x00"
        sh_entsize    = struct.pack("<Q", self._calc_entsize(name, int_sh_type, 64))
   
        data = sh_name + sh_type + sh_flags + sh_addr + sh_offset + sh_size
        data = data + sh_link + sh_info + sh_addralign + sh_entsize
 
        if len(data) != 64:
            debug.error("Broken section building! %d" % len(data))

        return data

    def _make_sect_header_32(self, name, address, size, file_off, strtab_idx, symtab_idx):
        #print "trying: %-30s | %.16x | %.8x | %d" % (name, address, size, file_off)

        int_sh_type = self._calc_sect_type(name)

        sh_name       = struct.pack("<I", self._calc_sect_name_idx(name))
        sh_type       = struct.pack("<I", int_sh_type)
        sh_flags      = struct.pack("<I", self._calc_sect_flags(name))
        sh_addr       = struct.pack("<I", address)
        sh_offset     = struct.pack("<I", file_off)
        sh_size       = struct.pack("<I", size)
        sh_link       = struct.pack("<I", self._calc_link(name, strtab_idx, symtab_idx, int_sh_type))
        sh_info       = "\x00" * 4 
        sh_addralign  = "\x01\x00\x00\x00"
        sh_entsize    = struct.pack("<I", self._calc_entsize(name, int_sh_type, 32))
   
        data = sh_name + sh_type + sh_flags + sh_addr + sh_offset + sh_size
        data = data + sh_link + sh_info + sh_addralign + sh_entsize
 
        if len(data) != 40:
            debug.error("Broken section building! %d" % len(data))

        return data

    def _null_sect_hdr(self, sz):
        return "\x00" * sz

    # the shstrtab section is "\x00\x2e" + section name for each section
    def _calc_string_data(self, module):
        data = ""

        for sect in module.get_sections():
            data = data + "\x00" + str(sect.sect_name)

        # put in our added section name + null terminator for section
        data = data + "\x00.shstrtab" + "\x00"

        return data              

    def _find_sec(self, sections_info, sym_addr):
        for sect in sections_info:
            (name, address, size) = sect
            
            if address <= sym_addr < address + size:
                #print "found in sec: %s" % name
                return name

        return ""

    def _fix_sym_table(self, module, sections_info):
        all_sym_data = ""
        
        first_name = False

        print "walking %d syms to be fixed...." % module.num_symtab

        if self.addr_space.profile.metadata.get('memory_model', '32bit') == '64bit':
            sym_type     = "elf64_sym"
            st_value_fmt = "<Q"
            st_size_fmt  = "<Q"
        else:
            sym_type     = "elf32_sym"
            st_value_fmt = "<I"
            st_size_fmt  = "<I"
              
        val_map      = {}
        name_idx_map = {}
        syms = obj.Object(theType="Array", targetType=sym_type, count=module.num_symtab, vm = module.obj_vm, offset = module.symtab)
        for (e, sym) in enumerate(syms):
            if sym.st_value > 0 and not module.obj_vm.profile.get_symbol_by_address("kernel", sym.st_value):
                val_map[sym.st_value.v()] = self._find_sec(sections_info, sym.st_value) 
                

        for (i, sect) in enumerate(module.get_sections()):
            name_idx_map[str(sect.sect_name)] = (i + 1, sect.address) ### account for null segment
      
        syms = obj.Object(theType="Array", targetType=sym_type, count=module.num_symtab, vm = module.obj_vm, offset = module.symtab)
        for sym in syms:
            # fix absolute addresses  
            st_value_int = sym.st_value.v()
            if st_value_int > 0 and st_value_int in val_map:
                secname = val_map[st_value_int]
                if secname in name_idx_map:
                    sect_addr = name_idx_map[secname][1]
                    # LOOK_HERE
                    st_value_sub  = st_value_int - sect_addr 
                    st_value_full = st_value_int
            
            else:
                st_value_sub  = st_value_int
                st_value_full = st_value_int
            
            st_value = struct.pack(st_value_fmt, st_value_sub)

            #### fix bindings ####
                
            # moved out of the sections part
            if sym.st_name > 0:
                first_name = True   
         
            if first_name:
                bind = 1 # STB_GLOBAL

                if sym.st_value == 0:
                    stype = 0

                elif module.obj_vm.profile.get_symbol_by_address("kernel", sym.st_value):
                    stype = 0 # STT_NOTYPE

                else:
                    secname = val_map[sym.st_value.v()]

                    # a .text. section but not relocations
                    if secname.find(".text") != -1 and secname.find(".rela") == -1:
                        stype = 2 # STT_FUNC                        
                    else:
                        stype = 1 # STT_OBJECT
            else: 
                bind  = 0 # STB_LOCAL
                stype = 3 # STT_SECTION

            b = (bind << 4) & 0xf0
            t = stype & 0xf
            st_info = (b | t) & 0xff
            #print "st_info: %x : %x | %x || %d | %x" % (sym.st_value, b, t, st_info, st_info)
            st_info = struct.pack("B", st_info)
            
            #### fix indexes ####
            if sym.st_value > 0 and sym.st_value.v() in val_map:
                secname = val_map[sym.st_value.v()]
                if secname in name_idx_map:
                    st_shndx = name_idx_map[secname][0]
                    st_shndx = struct.pack("<H", st_shndx)
                elif not secname:
                    st_shndx = struct.pack("<H", sym.st_shndx)
                else:
                    debug.error("no index for %s" % secname)
            else:
                st_shndx = struct.pack("<H", sym.st_shndx)

            ######

            # ones that aren't mangled
            st_name  = struct.pack("<I", sym.st_name)
            st_other = struct.pack("B", sym.st_other)
            st_size  = struct.pack(st_size_fmt, sym.st_size)        
   
            if sym_type == "elf64_sym": 
                sec_all = st_name + st_info + st_other + st_shndx + st_value + st_size
                sec_len = 24 

            else:
                sec_all = st_name + st_value + st_size + st_info + st_other + st_shndx
                sec_len = 16

            if len(sec_all) != sec_len:
                debug.error("Invalid section length: %d" % len(sec_all))

            all_sym_data = all_sym_data + sec_all        

        return all_sym_data

    def _get_module_data(self, module):
        (updated_sections, symtab_idx, load_addr) = self._parse_sections(module)
        
        if self.addr_space.profile.metadata.get('memory_model', '32bit') == '64bit':
            hdr_sz = 64
            sect_sz = 64
            _get_header       = self._get_header_64
            _make_sect_header = self._make_sect_header_64
        else:
            hdr_sz = 52
            _get_header       = self._get_header_32
            _make_sect_header = self._make_sect_header_32
            sect_sz = 40

        section_headers = self._null_sect_hdr(sect_sz)
        section_data    = ""

        strtab_idx = len(updated_sections) 

        print "******************"

        for (i, (name, address, size, file_off, sect_data)) in enumerate(updated_sections):
            section_headers = section_headers + _make_sect_header(name, address, size, file_off, strtab_idx, symtab_idx)
            
            print "added section %30s with size %6d file offset %6d len(section_data): %6d"  % (name, size, file_off, len(section_data))

            section_data    = section_data + sect_data              

            last_file_off = file_off
            last_sec_sz   = len(sect_data)

            if len(sect_data) != size:
                print "BROKEN SIZE"
                exit()

        # we need this section, but its not in memory
        # we can manually create it though and add it to our file
        sdata = self._calc_string_data(module)
        section_headers = section_headers + _make_sect_header(".shstrtab", 0, len(sdata), last_file_off + last_sec_sz, strtab_idx, symtab_idx)
        section_data = section_data + sdata

        # we stick it at the end
        num_sects  = len(updated_sections) + 1
            
        header = _get_header(load_addr - hdr_sz, hdr_sz + len(section_data), num_sects)

        return header + section_data + section_headers 

    def render_text(self, outfd, data):
        if not self._config.DUMP_DIR:
            debug.error("You must supply a --dump-dir output directory")
        
        for module in data:
            ## TODO: pass module.name through a char sanitizer 
            file_name = "{0}.{1:#x}.lkm".format(module.name, module.obj_offset)
            mod_file = open(os.path.join(self._config.DUMP_DIR, file_name), 'wb')
            mod_data = self._get_module_data(module)
            mod_file.write(mod_data)
            mod_file.close()
            outfd.write("Wrote {0} bytes to {1}\n".format(len(mod_data), file_name))




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_lsmod(common.AbstractMacCommand):
    """ Lists loaded kernel modules """

    def __init__(self, config, *args, **kwargs):
        common.AbstractMacCommand.__init__(self, config, *args, **kwargs)
        
        config.add_option('ADDR', short_option = 'a', default = None, help = 'Show info on VAD at or containing this address', action = 'store', type = 'int')

    def calculate(self):
        common.set_plugin_members(self)

        p = self.addr_space.profile.get_symbol("_kmod")
        kmodaddr = obj.Object("Pointer", offset = p, vm = self.addr_space)
        kmod = kmodaddr.dereference_as("kmod_info") 

        while kmod.is_valid():
            if not self._config.ADDR or (kmod.address <= self._config.ADDR <= (kmod.address + kmod.m("size"))):
                yield kmod
            kmod = kmod.next

    def unified_output(self, data):
        return TreeGrid([("Offset (V)", Address),
                        ("Module Address", Address),
                        ("Size", int),
                        ("Refs", int),
                        ("Version", str),
                        ("Name", str),
                        ], self.generator(data))
    def generator(self, data):
        for kmod in data:
            yield (0, [
                    Address(kmod.obj_offset),
                    Address(kmod.address),
                    int(kmod.m('size')),
                    int(kmod.reference_count),
                    str(kmod.version),
                    str(kmod.name),
                    ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"),
                                  ("Module Address", "[addrpad]"), 
                                  ("Size", "8"), 
                                  ("Refs", "^8"),
                                  ("Version", "12"),  
                                  ("Name", "")])
        for kmod in data:
            self.table_row(outfd,
                           kmod, 
                           kmod.address, 
                           kmod.m('size'), 
                           kmod.reference_count, 
                           kmod.version, 
                           kmod.name)



# Poison Ivy RAT detection and analysis for Volatility 2.0
#
# Version 1.0 (for release at the FIRST Conference, June 18, 2012)
#
# This version is limited to PoisonIvy's server version 2.3.1 
#
# Author: Andreas Schuster <a.schuster@forensikblog.de>
#
# This plugin is based on zeusscan2.py by Michael Hale Ligh.
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.taskmods as taskmods
import volatility.obj as obj
import volatility.win32.tasks as tasks
import volatility.utils as utils
import volatility.debug as debug
import volatility.plugins.malware.malfind as malfind

try:
    import yara
    has_yara = True
except ImportError:
    has_yara = False

class PIHOST(obj.CType):
    """Class for Poison Ivy Host/Proxy"""

    def next(self):
        """The next variable-length structure in the array"""

        return obj.Object("PIHOST", 
                         offset = self.obj_offset + self.length + 
                         self.obj_vm.profile.get_obj_size("PIHOST"), 
                         vm = self.obj_vm)

class PICONFIG(obj.CType):
    """Class for Poison Ivy Configuration Block"""

    def _read_hosts(self, memb):
        """Parse C2 or proxy config from data block"""

        # The first object in the array 
        host = obj.Object("PIHOST", offset = memb.obj_offset, vm = self.obj_vm)
        # The number of objects shall not exceed
        # the total size of the array 
        size = self.obj_vm.profile.get_obj_size("HOSTCFGSPACE")

        while (host.length > 0 and 
                    host.obj_offset < memb.obj_offset + size):
            yield host 
            host = host.next()

    def get_hosts(self):
        """Return the list of C2 hosts"""

        if self.ProxyCfgPresent == 1:
            return self._read_hosts(self.C2WhenProxy)
        else:
            return self._read_hosts(self.NextHop)

    def get_proxies(self):
        """Return the list of proxies"""        

        if self.ProxyCfgPresent == 1:
            return self._read_hosts(self.NextHop)
        else:
            raise StopIteration 

    @property
    def CopyDestFile(self):
        """Return the destination directory and file name"""

        destination = ''
        if self.CopyDestDir == 1:
            destination = '%WINDIR%'
        elif self.CopyDestDir == 2:
            destination = '%WINDIR%\\System32'

        if self.CopyAsADS == 1:
            destination += ':'
        else:
            destination += '\\' 

        return destination + self.m('CopyDestFile')

class PoisonIvyTypesx86(obj.ProfileModification):
    """Modification for Poison Ivy"""

    conditions = {'os': lambda x: x == 'windows', 
                  'memory_model': lambda x: x == '32bit'}

    def modification(self, profile):

        profile.object_classes.update({'PIHOST': PIHOST, 'PICONFIG': PICONFIG})

        profile.vtypes.update({
            'PIHOST': [ 4, { # minimum size based on the static fields 
                'length'                       : [ 0, ['unsigned char']], 
                'hostname'                     : [ 1, ['String', dict(length = lambda x : x.length)]], 
                'proto'                        : [ lambda x: x.obj_offset + 1 + x.length, ['Enumeration', dict(target = 'unsigned char', choices = {0: 'direct', 1: 'SOCKS', 2: 'HTTP'})]], 
                'port'                         : [ lambda x: x.obj_offset + 1 + x.length + 1, ['unsigned short']], 
            }], 
            'HOSTCFGSPACE' : [ 256, {
            }],
            'PICONFIG' : [ 0xf74, {
                'imp_socket'                   : [ 0x001, ['unsigned int']],
                'imp_connect'                  : [ 0x005, ['unsigned int']],
                'imp_closesocket'              : [ 0x009, ['unsigned int']],
                'imp_send'                     : [ 0x00d, ['unsigned int']],
                'imp_recv'                     : [ 0x011, ['unsigned int']],
                'imp_htons'                    : [ 0x015, ['unsigned int']],
                'imp_inet_addr'                : [ 0x019, ['unsigned int']],
                'imp_gethostbyname'            : [ 0x01d, ['unsigned int']],
                'imp_VirtualAlloc'             : [ 0x021, ['unsigned int']],
                'imp_VirtualFree'              : [ 0x025, ['unsigned int']],
                'imp_CreateThread'             : [ 0x029, ['unsigned int']],
                'imp_CreateProcessA'           : [ 0x02d, ['unsigned int']],
                'imp_RegCloseKey'              : [ 0x031, ['unsigned int']],
                'imp_RegOpenKeyExA'            : [ 0x035, ['unsigned int']],
                'imp_RegQueryValueExA'         : [ 0x039, ['unsigned int']],
                'imp_RegSetValueExA'           : [ 0x03d, ['unsigned int']],
                'imp_RegDeleteKeyA'            : [ 0x041, ['unsigned int']],
                'imp_RegCreateKeyExA'          : [ 0x045, ['unsigned int']],
                'imp_RegQueryInfoKeyA'         : [ 0x049, ['unsigned int']],
                'imp_RegEnumKeyExA'            : [ 0x04d, ['unsigned int']],
                'imp_DeleteFileA'              : [ 0x051, ['unsigned int']],
                'imp_CopyFileA'                : [ 0x055, ['unsigned int']],
                'imp_CreateFileA'              : [ 0x059, ['unsigned int']],
                'imp_GetKeyNameTextA'          : [ 0x05d, ['unsigned int']],
                'imp_GetActiveWindow'          : [ 0x061, ['unsigned int']],
                'imp_GetWindowTextA'           : [ 0x065, ['unsigned int']],
                'imp_WriteFile'                : [ 0x069, ['unsigned int']],
                'imp_CallNextHookEx'           : [ 0x06d, ['unsigned int']],
                'imp_SetFilePointer'           : [ 0x071, ['unsigned int']],
                'imp_ToAscii'                  : [ 0x075, ['unsigned int']],
                'imp_GetKeyboardState'         : [ 0x079, ['unsigned int']],
                'imp_GetLocalTime'             : [ 0x07d, ['unsigned int']],
                'imp_lstrcat'                  : [ 0x081, ['unsigned int']],
                'imp_CreateMutexA'             : [ 0x085, ['unsigned int']],
                'imp_RtlGetLastWin32Error'     : [ 0x089, ['unsigned int']],
                'imp_GetFileTime'              : [ 0x08d, ['unsigned int']],
                'imp_SetFileTime'              : [ 0x091, ['unsigned int']],
                'imp_OpenProcess'              : [ 0x095, ['unsigned int']],
                'imp_select'                   : [ 0x099, ['unsigned int']],
                'imp_LoadLibraryA'             : [ 0x09d, ['unsigned int']],
                'imp_CloseHandle'              : [ 0x0a1, ['unsigned int']],
                'imp_Sleep'                    : [ 0x0a5, ['unsigned int']],
                'imp_RtlMoveMemory'            : [ 0x0a9, ['unsigned int']],
                'imp_RtlZeroMemory'            : [ 0x0ad, ['unsigned int']],
                'imp_VirtualAllocEx'           : [ 0x0b1, ['unsigned int']],
                'imp_WriteProcessMemory'       : [ 0x0b5, ['unsigned int']],
                'imp_CreateToolhelp32Snapshot' : [ 0x0b9, ['unsigned int']],
                'imp_Process32First'           : [ 0x0bd, ['unsigned int']],
                'imp_Process32Next'            : [ 0x0c1, ['unsigned int']],
                'func_FindProcess'             : [ 0x0c5, ['unsigned int']],
                'imp_CreateRemoteThread'       : [ 0x0c9, ['unsigned int']],
                'imp_lstrcmpi'                 : [ 0x0cd, ['unsigned int']],
                'func_WriteProcess'            : [ 0x0d1, ['unsigned int']],
                'func_Extra'                   : [ 0x0d5, ['unsigned int']],
                'func_Main'                    : [ 0x0d9, ['unsigned int']],
                'func_GetProcAddressByCRC32'   : [ 0x0dd, ['unsigned int']],
                'NULL'                         : [ 0x0e1, ['unsigned int']],
                'func_Send_Receive'            : [ 0x0e5, ['unsigned int']],
                'func_strcopy'                 : [ 0x0e9, ['unsigned int']],
                'func_Keylogger_Hook'          : [ 0x0ed, ['unsigned int']],
                'func_0f1'                     : [ 0x0f1, ['unsigned int']],
                'func_Persist_ActiveSetup'     : [ 0x0f5, ['unsigned int']],
                'func_Injector'                : [ 0x0f9, ['unsigned int']],
                'func_Key_expansion'           : [ 0x0fd, ['unsigned int']],
                'func_Encrypt'                 : [ 0x101, ['unsigned int']],
                'func_Decrypt'                 : [ 0x105, ['unsigned int']],
                'func_Camellia_feistel'        : [ 0x109, ['unsigned int']],
                'func_Camellia_local'          : [ 0x10d, ['unsigned int']],
                'var_Camellia_Sbox1'           : [ 0x111, ['unsigned int']],
                'var_Camellia_Table1'          : [ 0x115, ['unsigned int']],
                'var_Camellia_Table2'          : [ 0x119, ['unsigned int']],
                'var_Socket'                   : [ 0x121, ['unsigned int']],
                'var_Version'                  : [ 0x129, ['unsigned int']],
                'CopyDestFile'                 : [ 0x12d, ['String', dict(length = 24)]],
                'Secret'                       : [ 0x145, ['String', dict(length = 32)]],
                'PersistActiveSetupGUID'       : [ 0x165, ['String', dict(length = 38)]],
                'NextHop'                      : [ 0x190, ['HOSTCFGSPACE']],
                'ProxyCfgPresent'              : [ 0x2c1, ['int']],
                'C2WhenProxy'                  : [ 0x2c5, ['HOSTCFGSPACE']],
                'PersistActiveSetupOn'         : [ 0x3f6, ['unsigned char']],
                'CopyDestDir'                  : [ 0x3f7, ['unsigned char']],
                'Melt'                         : [ 0x3f8, ['unsigned char']],
                'InjectPersist'                : [ 0x3f9, ['unsigned char']],
                'Keylogger'                    : [ 0x3fa, ['unsigned char']],
                'Mutex'                        : [ 0x3fb, ['String', dict(length = 20)]],
                'PersistActiveSetupName'       : [ 0x40f, ['String', dict(length = 9)]],
                'DefaultBrowserKey'            : [ 0x418, ['String', dict(length = 41)]],
                'InjectIntoDefaultBrowser'     : [ 0x441, ['unsigned char']],
                'InjectIntoProcessName'        : [ 0x442, ['String', dict(length = 20)]],
                'PersistActiveSetupKeyPart'    : [ 0x456, ['String', dict(length = 93)]],
                'PersistActiveSetupKey'        : [ 0x4b3, ['String', dict(length = 255)]],
                'CurrentFile'                  : [ 0x5b2, ['String', dict(length = 255)]],
                'PersistentFile'               : [ 0x6b1, ['String', dict(length = 255)]],
                'KeyloggerLogfile'             : [ 0x7b0, ['String', dict(length = 255)]],
                'IsAdmin'                      : [ 0x8af, ['unsigned char']],
                'var_8b4'                      : [ 0x8b4, ['unsigned int']],
                'var_8b8'                      : [ 0x8b8, ['unsigned char']],
                'var_8b9'                      : [ 0x8b9, ['unsigned int']],
                'KeyloggerTID'                 : [ 0x8bd, ['unsigned int']],
                'InjectorTID'                  : [ 0x8c1, ['unsigned int']],
                'hnd_Mutex'                    : [ 0x8c5, ['unsigned int']],
                'Challenge'                    : [ 0x8d9, ['array', 136, ['unsigned char']]],
                'var_Camellia_Keyschedule'     : [ 0x96b, ['array', 272, ['unsigned char']]],
                'hnd_Kernel32'                 : [ 0xabb, ['unsigned int']],
                'hnd_User32'                   : [ 0xabf, ['unsigned int']],
                'hnd_Ws2_32'                   : [ 0xac3, ['unsigned int']],
                'hnd_Advapi32'                 : [ 0xad3, ['unsigned int']],
                'hnd_Ntdll'                    : [ 0xadb, ['unsigned int']],
                'imp_lstrlenA'                 : [ 0xaf0, ['unsigned int']],
                'UseProxy'                     : [ 0xaf4, ['unsigned char']],
                'ProxyNoPersist'               : [ 0xaf5, ['unsigned char']],
                'func_af6'                     : [ 0xaf6, ['unsigned int']],
                'ServerId'                     : [ 0xafa, ['String', dict(length = 255)]],
                'ServerGroup'                  : [ 0xbf9, ['String', dict(length = 255)]],
                'imp_GetFileSize'              : [ 0xcf8, ['unsigned int']],
                'imp_ReadFile'                 : [ 0xcfc, ['unsigned int']],
                'func_CopyFile'                : [ 0xd00, ['unsigned int']],
                'func_Inject_explorer'         : [ 0xd04, ['unsigned int']],
                'Inject'                       : [ 0xd08, ['unsigned char']],
                'PersistHKLMRun'               : [ 0xd09, ['unsigned char']],
                'func_Persist_HKLMRun'         : [ 0xd0a, ['unsigned int']],
                'func_Keylogger'               : [ 0xd0e, ['unsigned int']],
                'CopyAsADS'                    : [ 0xd12, ['unsigned char']],
                'PersistHKLMRunName'           : [ 0xe12, ['String', dict(length = 255)]]
            }]
        })

# This simple signature is based on string constants. It it easy to find, easy
# to explain - and easy to break! Therefore you're advised to develop robust, 
# code based signatures for daily work.
signatures = {
    'namespace1' : 'rule pivars {strings: $a = { \
        53 74 75 62 50 61 74 68 ?? 53 4F 46 54 57 41 52\
        45 5C 43 6C 61 73 73 65 73 5C 68 74 74 70 5C 73\
        68 65 6C 6C 5C 6F 70 65 6E 5C 63 6F 6D 6D 61 6E\
        64 [22] 53 6F 66 74 77 61 72 65 5C 4D 69 63 72 6F\
        73 6F 66 74 5C 41 63 74 69 76 65 20 53 65 74 75\
        70 5C 49 6E 73 74 61 6C 6C 65 64 20 43 6F 6D 70\
        6F 6E 65 6E 74 73 5C } condition: $a}'
}

class PoisonIvyScan(taskmods.DllList):
    "Detect processes infected with Poison Ivy"

    @staticmethod
    def is_valid_profile(profile):
        return (profile.metadata.get('os', 'unknown') == 'windows' and
                profile.metadata.get('memory_model', '32bit') == '32bit')

    def get_vad_base(self, task, address):
        """ Get the VAD starting address """        

        for vad in task.VadRoot.traverse():
            if address >= vad.Start and address < vad.End:
                return vad.Start

        # This should never really happen
        return None

    def calculate(self):

        if not has_yara:
            debug.error("Yara must be installed for this plugin")

        addr_space = utils.load_as(self._config)
        
        if not self.is_valid_profile(addr_space.profile):
            debug.error("This command does not support the selected profile.")
        
        rules = yara.compile(sources = signatures)

        for task in self.filter_tasks(tasks.pslist(addr_space)):
            scanner = malfind.VadYaraScanner(task = task, rules = rules)

            for hit, address in scanner.scan():
                vad_base_addr = self.get_vad_base(task, address)
                if address - vad_base_addr > 0x1000:
                    continue

                yield task, vad_base_addr

    def render_text(self, outfd, data):

        self.table_header(outfd, [("Name", "20"), 
                                  ("PID", "8"),
                                  ("Data VA", "[addrpad]")])

        for task, start in data:
            self.table_row(outfd, task.ImageFileName, task.UniqueProcessId, start)

class PoisonIvyConfig(PoisonIvyScan):
    "Locate and parse the Poison Ivy configuration"

    def render_text(self, outfd, data):
        
        delim = '-' * 80

        for task, start in data:

            outfd.write("{0}\n".format(delim))
            
            proc_addr_space = task.get_process_address_space()

            config = obj.Object('PICONFIG', offset = start, vm = proc_addr_space)

            outfd.write('Process: {0} ({1})\n\n'.format(task.ImageFileName, task.UniqueProcessId))
            outfd.write('Infection:\n')

            if config.IsAdmin == 1:
                outfd.write('\tPoisonIvy has ADMIN privileges!\n')
            else:
                outfd.write('\tPoisonIvy has user privileges.\n')

            outfd.write('\tVersion: {0}\n'.format(config.var_Version))
            outfd.write('\tBase VA: {0:#x}\n'.format(config.func_Main))
            outfd.write('\tExtra VA: {0:#x}\n'.format(config.func_Extra))
            outfd.write('\tData VA: {0:#x}\n'.format(start))
            outfd.write('\tMutex: {0}\n'.format(config.Mutex))
            outfd.write('\tOriginal file: {0}\n'.format(config.CurrentFile))
            outfd.write('\tMelt original file: {0}\n\n'.format(config.Melt == 1))

            outfd.write("Command and Control:\n")
            for i, host in enumerate(config.get_hosts()):
                outfd.write('\tHost {0}: {1}:{2} ({3})\n'.format(i, host.hostname, host.port, host.proto))

            # secret (either password or keyfile)
            if config.Secret.isalnum():
                outfd.write('\tPassword: {0}\n'.format(config.Secret))
            else:
                outfd.write('\tKey (from file): 0x{0}'.format(config.Secret.encode('hex')))

            # management info
            outfd.write('\tId: {0}\n'.format(config.ServerId))
            outfd.write('\tGroup: {0}\n\n'.format(config.ServerGroup))

            outfd.write('Keylogger:\n')
            outfd.write('\tKeylogger: {0}\n'.format(config.Keylogger == 1))
            if config.Keylogger == 1:
                outfd.write('\tKeylogger TID: {0}\n'.format(config.KeyloggerTID))
                outfd.write('\tKeylogger Setup: {0:#x}\n'.format(config.func_Keylogger))
                outfd.write('\tKeylogger Routine: {0:#x}\n'.format(config.func_Keylogger_Hook))
                outfd.write('\tKeylogger logfile: {0}\n'.format(config.KeyloggerLogfile))

            outfd.write("\nCopy file:\n")
            outfd.write('\tCopy routine: {0:#x}\n'.format(config.func_CopyFile))
            outfd.write('\tDestination: {0}\n'.format(config.CopyDestFile))

            outfd.write("\nPersistence:\n")
            outfd.write('\tActive Setup: {0}\n'.format(config.PersistActiveSetupOn == 1))
            if config.PersistActiveSetupOn == 1:
                outfd.write('\tActive Setup key: {0}\n'.format(config.PersistActiveSetupKey))
                outfd.write('\tActive Setup name: {0}\n'.format(config.PersistActiveSetupName))
                outfd.write('\tSetup routine: {0:#x}\n'.format(config.func_Persist_ActiveSetup))

            outfd.write("\tHKLM Run: {0}\n".format(bool(config.PersistHKLMRun == 1)))
            if config.PersistHKLMRun == 1:
                outfd.write("\tHKLM Run name: {0}\n".format(config.PersistHKLMRunName))
                outfd.write("\tSetup routine: {0:#x}\n".format(config.func_Persist_HKLMRun))

            outfd.write("\nInjector:\n")
            outfd.write("\tInject into other processes: {0}\n".format(config.Inject == 1))
            if config.Inject == 1:
                outfd.write('\tPersistently: {0}\n'.format(config.InjectPersist == 1))
                outfd.write('\tInjector TID: {0}\n'.format(config.InjectorTID))
                outfd.write('\tInjector Routine: {0:#x}\n'.format(config.func_Injector))
                outfd.write('\tTarget process name: {0}\n'.format(config.InjectIntoProcessName))
                outfd.write('\tTarget default browser: {0}\n'.format(config.InjectIntoDefaultBrowser == 1))

            outfd.write("\nProxy:\n")
            outfd.write('\tUse Proxy: {0}\n'.format(config.UseProxy == 1))
            if config.UseProxy == 1:
                outfd.write('\tPersistently: {0}\n'.format(config.ProxyNoPersist == 0))
                for i, proxy in enumerate(config.get_proxies()):
                    outfd.write('\tHost {0}: {1}:{2} ({3})\n'.format(i, proxy.hostname, proxy.port, proxy.proto))

            outfd.write("\nDecrypt: {0:#x}\n".format(config.func_Decrypt))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.ifconfig as linux_ifconfig
import volatility.plugins.linux.pslist as linux_pslist
import volatility.debug as debug
import volatility.obj as obj

class linux_list_raw(linux_common.AbstractLinuxCommand):
    """List applications with promiscuous sockets"""

    def __init__(self, config, *args, **kwargs):
        self.fd_cache = {}
        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)

    def _SOCK_INODE(self, sk):
        backsize = self.profile.get_obj_size("socket")
        addr = sk + backsize

        return obj.Object('inode', offset = addr, vm = self.addr_space) 

    def _walk_net_spaces(self):
        offset = self.addr_space.profile.get_obj_offset("sock_common", "skc_node")
        
        nslist_addr = self.addr_space.profile.get_symbol("net_namespace_list")
        nethead = obj.Object("list_head", offset = nslist_addr, vm = self.addr_space)
            
        for net in nethead.list_of_type("net", "list"):
            node = net.packet.sklist.first.dereference().v()
            
            sk = obj.Object("sock", offset = node - offset, vm = self.addr_space)

            while sk.is_valid():
                inode = self._SOCK_INODE(sk.sk_socket)

                ino = inode

                yield ino

                sk = obj.Object("sock", offset = sk.sk_node.next - offset, vm = self.addr_space)

    def _fill_cache(self):
        for task in linux_pslist.linux_pslist(self._config).calculate():
            for filp, fd in task.lsof():
                filepath = linux_common.get_path(task, filp)
                if type(filepath) == str and filepath.find("socket:[") != -1:
                    to_add = filp.dentry.d_inode.i_ino.v()
                    self.fd_cache[to_add] = [task, filp, fd, filepath]
   
    def _find_proc_for_inode(self, inode):
        if self.fd_cache == {}:
            self._fill_cache()
        
        inum = inode.i_ino.v()

        if inum in self.fd_cache:
            (task, filp, fd, filepath) = self.fd_cache[inum]
        else:
            (task, filp, fd, filepat)  = (None, None, None, None)

        return (task, fd, inum)

    def __walk_hlist_node(self, node):    
        seen = set()

        offset = self.addr_space.profile.get_obj_offset("sock_common", "skc_node")

        nxt = node.next.dereference()
        
        while nxt.is_valid() and nxt.obj_offset not in seen:
            ## Instantiate the object
            item = obj.Object(obj_type, offset = nxt.obj_offset - offset, vm = self.addr_space)

            seen.add(nxt.obj_offset)

            yield item

            nxt = nxt.next.dereference()

    def _walk_packet_sklist(self):
        sklist_addr = self.addr_space.profile.get_symbol("packet_sklist")
    
        sklist = obj.Object("hlist_head", offset = sklist_addr, vm = self.addr_space)

        for sk in self.__walk_hlist_node(sklist.first):
            yield self._SOCK_INODE(sk.sk_socket)

    def calculate(self):
        linux_common.set_plugin_members(self)

        sym_addr = self.addr_space.profile.get_symbol("packet_sklist") 

        # old kernels before namespaces
        if sym_addr:
            for inode in self._walk_packet_sklist():
                yield self._find_proc_for_inode(inode) 
        else:
            for inode in self._walk_net_spaces():
                yield self._find_proc_for_inode(inode)
            
    def render_text(self, outfd, data):

        self.table_header(outfd, [("Process", "16"),
                                  ("PID", "6"),
                                  ("File Descriptor", "5"),
                                  ("Inode", "18"),
                                 ])

        for (task, fd, inum) in data:
            if task:
                self.table_row(outfd, task.comm, task.pid, fd, inum)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import struct, string
import volatility.obj as obj
import volatility.debug as debug
import volatility.addrspace as addrspace
import volatility.plugins.mac.common  as mac_common
import volatility.plugins.mac.pstasks as mac_tasks
from volatility.renderers import TreeGrid

bash_vtypes = {
    'bash32_hist_entry': [ 0xc, {
    'line': [0x0, ['pointer', ['String', dict(length = 1024)]]],
    'timestamp': [0x4, ['pointer', ['String', dict(length = 1024)]]],
    'data': [0x8, ['pointer', ['void']]],
    }],
    
    'bash64_hist_entry': [ 24, {
    'line': [0, ['pointer', ['String', dict(length = 1024)]]],
    'timestamp': [8, ['pointer', ['String', dict(length = 1024)]]],
    'data': [16, ['pointer', ['void']]],
    }],
}

class _mac_hist_entry(obj.CType):
    """A class for history entries"""

    def is_valid(self):
        line_addr = self.line_ptr()        
        time_addr = self.time_ptr() 
 
        if (not obj.CType.is_valid(self) or  
                    not self.obj_vm.is_valid_address(line_addr) or 
                    not self.obj_vm.is_valid_address(time_addr)):
            return False

        ts = self.obj_vm.read(time_addr, 256)
        if not ts:
            return False
    
        idx = ts.find("\x00")
        if idx != -1:
            ts = ts[:idx]

        # At this point in time, the epoc integer size will 
        # never be less than 10 characters, and the stamp is 
        # always preceded by a pound/hash character. 
        if len(ts) < 10 or str(ts)[0] != "#":
            return False

        # The final check is to make sure the entire string
        # is composed of numbers. Try to convert to an int. 
        try:
            int(str(ts)[1:])
        except ValueError:
            return False 

        return True

    def line(self):
        line_addr = self.line_ptr()
        buf = self.obj_vm.read(line_addr, 256)
        if buf:
            idx = buf.find("\x00")
            if idx != -1:
                buf = buf[:idx]  

            ret = "".join([c for c in buf if c in string.printable])
        else:
            ret = ""

        return ret

    @property
    def time_as_integer(self):
        # Get the string and remove the leading "#" from the timestamp
        time_addr = self.time_ptr()
        ts = self.obj_vm.read(time_addr, 256)
        ts = ts[1:] 
        idx = ts.find("\x00")
        if idx != -1:
            ts = ts[:idx]
 
        # Convert the string into an integer (number of seconds)
        return int(ts)

    def time_object(self):
        nsecs = self.time_as_integer
        # Build a timestamp object from the integer 
        time_val = struct.pack("<I", nsecs)
        time_buf = addrspace.BufferAddressSpace(self.obj_vm.get_config(), data = time_val)
        time_obj = obj.Object("UnixTimeStamp", offset = 0, vm = time_buf, is_utc = True)
        return time_obj

    def line_ptr(self):
        addr = self.m("line").obj_offset
        return self.read_ptr(addr)

    def time_ptr(self):
        addr = self.m("timestamp").obj_offset
        return self.read_ptr(addr)

class bash64_hist_entry(_mac_hist_entry):
    def read_ptr(self, addr):
        addr = self.obj_vm.read(addr, 8)
        addr = struct.unpack("<Q", addr)[0]
        return addr

class bash32_hist_entry(_mac_hist_entry):
    def read_ptr(self, addr):
        addr = self.obj_vm.read(addr, 4)
        addr = struct.unpack("<I", addr)[0]
        return addr

class MacBashTypes(obj.ProfileModification):
    conditions = {"os" : lambda x : x in ["mac"]}

    def modification(self, profile):
        profile.vtypes.update(bash_vtypes)
        profile.object_classes.update({"bash32_hist_entry": bash32_hist_entry, "bash64_hist_entry": bash64_hist_entry})

class mac_bash(mac_tasks.mac_tasks):
    """Recover bash history from bash process memory"""

    def __init__(self, config, *args, **kwargs): 
        mac_tasks.mac_tasks.__init__(self, config, *args, **kwargs)
        self._config.add_option('SCAN_ALL', short_option = 'A', default = False, help = 'scan all processes, not just those named bash', action = 'store_true')    

    def unified_output(self, data):
    
        return TreeGrid([("Pid", int), 
                            ("Name", str),
                            ("Command Time", str),
                            ("Command", str),
                            ], self.generator(data))
                            
    def generator(self, data):
        for task in data:
            if not (self._config.SCAN_ALL or str(task.p_comm) == "bash"):
                continue

            for hist_entry in task.bash_history_entries():
                yield (0, [
                    int(task.p_pid),
                    str(task.p_comm),
                    str(hist_entry.time_object()),
                    str(hist_entry.line()),
                    ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Pid", "8"), 
                                  ("Name", "20"),
                                  ("Command Time", "30"),
                                  ("Command", ""),])
                                    
        for task in data:
            if not (self._config.SCAN_ALL or str(task.p_comm) == "bash"):
                continue
            
            for hist_entry in task.bash_history_entries():
                self.table_row(outfd, task.p_pid, task.p_comm, 
                           hist_entry.time_object(), 
                           hist_entry.line())



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class mac_network_conns(common.AbstractMacCommand):
    """ Lists network connections from kernel network structures """

    # in_pcblookup_hash - bsd/netinet/in_pcb.c 
    def _walk_pcb_hash(self, proto_pcbinfo):
        pcb_hash = obj.Object("Array", offset = proto_pcbinfo.hashbase, vm = self.addr_space, targetType = "Pointer", count = proto_pcbinfo.hashmask + 1)

        for pcb_ent in pcb_hash:
            head = pcb_ent.dereference_as("inpcbhead")

            if not head:
                continue

            inpcb = head.lh_first.dereference_as("inpcb")

            while inpcb:
                yield inpcb
                inpcb = inpcb.inp_hash.le_next 

    # in_pcblookup_hash - bsd/netinet/in_pcb.c 
    def _walk_pcb_list(self, proto_pcbinfo):
        inpcb = proto_pcbinfo.listhead.lh_first.dereference_as("inpcb")

        while inpcb:
            yield inpcb
            inpcb = inpcb.inp_list.le_next 

    def _walk_pcb_entries(self, inpcbinfo_addr):
        pcbs = {}
        
        inpcbinfo = obj.Object("inpcbinfo", offset = inpcbinfo_addr, vm = self.addr_space)

        for pcbinfo in self._walk_pcb_list(inpcbinfo):
            pcbs[pcbinfo.obj_offset] = pcbinfo
    
        for pcbinfo in self._walk_pcb_hash(inpcbinfo):
            pcbs[pcbinfo.obj_offset] = pcbinfo

        for pcbinfo in pcbs.values():
            (lip, lport, rip, rport) = pcbinfo.ipv4_info() 
            yield (pcbinfo, lip, lport, rip, rport)

    def calculate(self):
        common.set_plugin_members(self)    

        entries = []
        
        tcbinfo_addr   = self.addr_space.profile.get_symbol("_tcbinfo")
        udbinfo_addr   = self.addr_space.profile.get_symbol("_udbinfo") 
        ripdbinfo_addr = self.addr_space.profile.get_symbol("_ripcbinfo")
        
        info_addrs = [("TCP", tcbinfo_addr), ("UDP", udbinfo_addr), ("RAW", ripdbinfo_addr)]

        for (proto_str, info_addr) in info_addrs:
            for (pcbinfo, lip, lport, rip, rport) in self._walk_pcb_entries(info_addr):
                if proto_str == "TCP":
                    state = pcbinfo.get_tcp_state()
                else:
                    state = ""
                yield (proto_str, pcbinfo, lip, lport, rip, rport, state)

    def unified_output(self, data):
        return TreeGrid([("Offset (V)", Address),
                                  ("Protocol", str),
                                  ("Local IP", str),
                                  ("Local Port", int),
                                  ("Remote IP", str),
                                  ("Remote Port", int),
                                  ("State", str),
                                 ], self.generator(data))

    def generator(self, data):
        for (proto, pcb, lip, lport, rip, rport, state) in data:
            yield(0, [
                Address(pcb.obj_offset),
                str(proto),
                str(lip),
                int(lport),
                str(rip),
                int(rport),
                str(state),
                ])

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"), 
                                  ("Protocol", "4"),
                                  ("Local IP", "20"),
                                  ("Local Port", "6"),
                                  ("Remote IP", "20"),
                                  ("Remote Port", "6"),
                                  ("State", ""),
                                 ])

        for (proto, pcb, lip, lport, rip, rport, state) in data: 
            self.table_row(outfd, pcb.obj_offset, proto, lip, lport, rip, rport, state)
